// FPdigits.S
//
// Converts a double-precision mantissa to a
// string of decimal digits.
//
																																		  
            #include    "ao32aaLibPrivate.inc"

            .text

// tenTo15: used to multiply a value between 1.0
// and 2.0 in order to convert the mantissa to
// an actual integer.

			.align		3
tenTo15:    .double     1.0e+15


ten:		.double		10.0


// potPos, potNeg, and expTbl.
//
// Power of tens tables (pot) used to quickly
// multiply or divide a floating-point value
// by powers of ten. expTbl is the power of
// ten exponent (absolute value) for each of 
// the entries in these tables.
            
potPos:     .double     1.0e+0
            .double     1.0e+1
            .double     1.0e+2
            .double     1.0e+4
            .double     1.0e+8
            .double     1.0e+16
            .double     1.0e+32
            .double     1.0e+64
            .double     1.0e+128
            .double     1.0e+256
expCnt      =           (.-potPos) / 8
            
potNeg:     .double     1.0e-0
            .double     1.0e-1
            .double     1.0e-2
            .double     1.0e-4
            .double     1.0e-8
            .double     1.0e-16
            .double     1.0e-32
            .double     1.0e-64
            .double     1.0e-128
            .double     1.0e-256
            
expTbl:     .word       0
            .word       1
            .word       2
            .word       4
            .word       8
            .word       16
            .word       32
            .word       64
            .word       128
            .word       256


   
////////////////////////////////////////////////////////////
//
// divMod56by10
//
//	Divides value passed in R1:R0 (54 actual bits) by 10.
//	Returns quotient in R2:R1, remainder in R3
//
//  Does this by multipling by 0.1 so no division is involved.
//
// see http://homepage.cs.uiowa.edu/~jones/bcd/divide.html
//
//	
// 64-bit multiply:
//
// 		  	  HO     LO
//  		  AAA : BBB (R1:R0)
//  		x CCC : DDD (0x00c...ccd)
//       -------------
//             | D*B |
//          | D*A |
//          | C*B |
//     + | C*A |
// -------------------
// | AAABBB * CCCDDD |
//
// Note: CCC:DDD =
//  0xcccccccc:0xcccccccd
// which is 0.1 in fixed-point
// notation.
	
	proc	divMod56by10, thumb, public
	push	{r0, r4-r11, lr}

	// R4:R5 = D*B
	
	ldr		r0, =0xcccccccd	// D
	umull	r4, r5, r1, r0
	
	// R6:R7 = D*A
	
	umull	r6, r7, r2, r0
	
	// R8:R9 = C*B
	
	ldr		r0, =0xcccccccc	// C
	umull	r8, r9, r1, r0	
	
	// R10:R11 = C*A
	
	umull	r10, r11, r2, r0
		
	// Now add together the partial
	// products. Note that the
	// LO word of the partial products
	// does not affect the result,
	// so this code doesn't add it in.
	
	adds	r5, r5, r6
	adcs	r7, r7, r9
	adc		r11, r11, #0
	
	adds	r5, r5, r8
	adcs	r7, r7, r10
	adcs	r11, r11, #0
		
	// R1:R0 * 10 * 0x1000_0000 * 8 is sitting in
	// R11:R7. Therefore, R10 holds the quotient * 8.
	
	lsr		r7, r7, #3
	orr		r7, r7, r11, lsl #29
	lsr		r11, r11, #3
	
	// Now, multiply the quotient by 10 and
	// subtract it from the original value (R2:R1)
	// to get the remainder:
	
	mov		r0, #10
	mls		r3, r7, r0, r1
	
	mov		r1, r7
	mov		r2, r11
	
	pop		{r0, r4-r11, pc}
	endp	divMod56by10
			
//////////////////////////////////////////////////////////////
//
// u53toStr:
//
//  Converts a 53-bit unsigned integer to a string containing
//  exactly 16 digits (technically, it does 64-bit arithmetic,
//  but is limited to 53 bits because of the 16-digit output
//  format).
//
//  Inputs:
//      R0:     pointer to buffer to receive string.
//      R2:R1-  unsigned 53-bit integer to convert.
//
//  Outputs:
//      Buffer: receives the zero-terminated string.
//      R0:     points at zero-terminating byte in string.
//
//  Buffer must have at least 17 bytes allocated for it.
//
// This code is a bit simplified from the u64toStr function
// due to the fact that it always emits exactly 16 digits
// (never any leading zeros).            


    proc    u53ToStr, thumb, public
    
	push	{r0-r3, lr}

	add		r0, r0, #16		// End of buffer
	mov		lr, #0			// Emit zero byte
	strb	lr, [r0], #-1
	
    // Handle LO digit here. Note that the LO
    // digit (digit #0) will ultimately be moved into
    // bit positions 56-63 of R5 because numeric
    // strings are, intrinsically, big endian (with
    // the HO digit appearing first in memory).
    
	bl		divMod56by10		// 64-bit R1:R0/10    	
    orr     r3, r3, #'0'		// Convert rem to digit
	strb	r3, [r0], #-1
	
    // The following is an unrolled loop
    // (for speed) that processes the
    // remaining 15 digits.
    //
    // Handle digits 1 to 3 here:
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
	     
    // Handle digits 5-8 here:
    
    mov     r5, #0
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
    // Handle digits 8 to 11 here:
    
    mov     r5, #0
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
	    
    // Handle digits 12 to 15 here:
    
    mov     r5, #0
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	bl		divMod56by10	  
    orr     r3, r3, #'0'	  
	strb	r3, [r0], #-1
    
	pop		{r0-r3, pc}
    endp    u53ToStr
         
///////////////////////////////////////////////////////////
//		 
// doubleToU64
//
//	Converts a double-precision value to a 64-bit integer.
//
// Input:
//	D0-		Double-precision value to convert.
//
// Output:
//	R2:R1-	64-bit integer result.

	proc		doubleToU64	 
	push		{r0, r3-r5, lr}

// The slow way:
//
//	vmov		r0, r1, d0
//	bl			__aeabi_d2ulz	// Double to 64-bit unsigned
//	mov			r2, r1
//	mov			r1, r0
//	pop			{r0, r3-r5, pc}

	// The faster way (only works for values in the range
	// 1e15 <= r2:r1 < 1e16).
	//
	//	 Begin by moving the value to convert into R2:R1
	// and extracting the exponent value from the double.

	vmov		r1, r2, d0
	lsr			r3, r2, #20		// Extract exponent
	ldr			r4, =0x7ff
	and			r3, r3, r4		// Remove sign
	
	// Remove the exponent and sign from the mantissa:
	
	ldr			r4, =0x000fffff
	and			r2, r2, r4
	orr			r2, r2, #0x00100000	// Implied bit
		
	// Values in the range 1e15 to 1e16 will have exponents
	// in the range 0x430 to 0x434. Need to treat 0x434
	// differently from the others:
	
	ldr			r4, = 0x433
	subs		r3, r4, r3		// Base expoent for 1E15
	bmi			is0x434			// Special case for 9.xxxE15
	beq			noShift

	// For exponents in the range 0x431 to 0x433, must shift
	// the mantissa to the right the number of bits computed
	// from exp-0x430:
	
shrLp:
	lsrs		r2, r2, #1
	rrx			r1, r1
	subs		r3, r3, #1
	bne			shrLp

noShift:	
	pop			{r0, r3-r5, pc}

// Special case when the exponent is 0x434; shift the
// mantissa one bit to the left rather than the right:

is0x434:
	adds		r1, r1, r1
	adc			r2, r2, r2
	pop			{r0, r3-r5, pc}
	endp		doubleToU64			
	
		    
            
/////////////////////////////////////////////////////////
//
// FPDigits:
//
//  Used to convert a floating-point value
//  in D0 to a string of digits.
//
// Inputs:
//      D0: Double-precision value to convert
//      R0: Pointer to buffer to receive chars
//
// Outputs:
//      R0: Still points at buffer
//      R1: Contains exponent of the number
//      R2: Contains sign (space or '-')
 
    proc    	FPDigits, thumb, public

	push		{r0, r3, r4, r5, r6, r7, r8, r9, r10, lr}
	vpush		{d0, d1, d2, d3, d4, d5}
		    
    // Save sign in R9 for the time being:
	
    mov     	r9, #' '        // Assume sign is +

#define fp1 d2 
    
	// D2 holds 1.0
	
    vmov.f64 fp1, #1.0
    
    // Special case for 0.0:
    
    vcmp.f64 	d0, #0.0
	vmrs		APSR_nzcv, FPSCR
    bne     	d0not0
    ldr     	r1, =0x30303030
    str     	r1, [r0]
    str     	r1, [r0, #4]
    str     	r1, [r0, #8]
    str     	r1, [r0, #12]
    mov     	r1, #0          // Exponent = 0
    strb    	r1, [r0, #16]   // Also zero terminator
	mov			r2, #' '		// Sign is positive
    b 	    	fpdDone
            
// If the number is non-zero, deal with it here. Note
// that the flags were set by comparing D0 to 0.0 earlier.

d0not0:     
	bge    	 fpIsPositive    // See if positive or negative
        
    // If negative, negate and change the sign
    // character to '-'
    
    vabs.f64    d0, d0
    mov     	r9, #'-'

// Get the number between 1.0 and <10.0 so we can figure out 
// what the exponent is.  Begin by checking to see if we have 
// a positive or negative exponent.

fpIsPositive:
    mov     	r1, #0         	// Initialize exponent
    vcmp.f64    d0, fp1
	vmrs		APSR_nzcv, FPSCR
    bge     	posExp
    
    // The value is between 0.0 and 1.0,
    // exclusive,  at this point.  That means this
    // number has a negative  exponent.  Multiply
    // the number by an appropriate power of ten
    // until we get it in the range 1 through 10.

    ldr     	r6, =expTbl
    mov     	r7, #expCnt

// Search through the potNeg table until we find a power
// of ten that is less than the value in D0:

cmpNegExp:          
    subs    	r7, r7, #1
    blt     	test1       // Branch if r7 < 1
    
    ldr     	r4, =potNeg
	add			r4, r4, r7, lsl #3
    vld1.f64   	{d1}, [r4]  // D1=potNeg[r7*8]
    vcmp.f64	d1, d0      // repeat while
	vmrs		APSR_nzcv, FPSCR
    ble     	cmpNegExp   // table <= value           
            
    // Eliminate the current exponent indexed by
    // r7 by multiplying by the corresponding
    // entry in potPos.

    ldr     	r8, [r6, r7, lsl #2] 	// r8=expTbl[r7*4]
    sub     	r1, r1, r8
	
    ldr     	r5, =potPos
	add			r5, r5, r7, lsl #3 
    vld1.f64	{d1}, [r5]  			// D1=potPos[r7*8]
    vmul.f64   	d0, d0, d1
    b       	cmpNegExp 
            
// If we get to this point, then we've indexed through
// all the elements in the potNeg and it's time to stop.
//
// If the remainder is *exactly* 1.0, then we can branch
// on to InRange1_10; otherwise, we still have to multiply
// by 10.0 because we've overshot the mark a bit.

test1:      
	vcmp.f64	d0, fp1
	vmrs		APSR_nzcv, FPSCR
    beq     	inRange1_10
    
	ldr			r3, =ten
    vldr.f64   	d1, [r3]
    vmul.f64	d0, d0, d1
    sub     	r1, r1, #1      // Decrement exponent
    b       	inRange1_10
	
            
            
//  At this point, we've got a number that is one or greater.
//  Once again, our task is to get the value between 1.0 and <10.0.
            
posExp:
    mov     	r7, #expCnt
    ldr     	r6, =expTbl
       
cmpPosExp:
	subs    	r7, r7, #1
    blt     	inRange1_10     	// if r7 < 1
    ldr     	r5, =potPos
	add			r5, r5, r7, lsl #3 
    vld1.f64    {d1}, [r5]			// D1=potPos[r7*8]
    vcmp.f64	d1, d0
	vmrs		APSR_nzcv, FPSCR
    bgt     	cmpPosExp
    
    ldr     	r8, [r6, r7, lsl #2] 			// r8=expTbl[r7*8]            
    add     	r1, r1, r8
    vdiv.f64   	d0, d0, d1
    b       	cmpPosExp
            

// Okay, at this point the number is in the range 1 <= x < 10,
// Let's multiply it by 1e+15 to put the most significant digit
// into the 16th print position.  Then convert the result to
// a string and store away in memory.
            
inRange1_10:
	mov			r10, r1			// Preserve exponent
    vldr.f64  	d1, tenTo15
    vmul.f64   	d0, d0, d1
	bl			doubleToU64
	
    // Convert the integer mantissa to a 
    // string of digits:
    
    bl      	u53ToStr
            
	mov			r2, r9	// Return sign in R2.
	mov			r1, r10 // Return exponent in R1.
	 
fpdDone:   
	vpop		{d0, d1, d2, d3, d4, d5}
	pop			{r0, r3, r4, r5, r6, r7, r8, r9, r10, pc}
    endp    	FPDigits
