// spToStr.S
//
// Floating-point (single) to string conversion.
//
// Provides both exponential (scientific notation)
// and decimal output formats.
//
																																		  
            #include    "ao32aaLibPrivate.inc"

            .text

// tenTo15: used to multiply a value between 1.0
// and 2.0 in order to convert the mantissa to
// an actual integer.

			.align		3
tenTo6:    .single     1.0e+6


ten:		.single		10.0
one:		.single		1.0


// potPos, potNeg, and expTbl.
//
// Power of tens tables (pot) used to quickly
// multiply or divide a floating-point value
// by powers of ten. expTbl is the power of
// ten exponent (absolute value) for each of 
// the entries in these tables.
            
potPos32:   .single     1.0e+0
            .single     1.0e+1
            .single     1.0e+2
            .single     1.0e+4
            .single     1.0e+8
            .single     1.0e+16
            .single     1.0e+32
expCnt      =           (.-potPos32) / 4
            
potNeg32:   .single     1.0e-0
            .single     1.0e-1
            .single     1.0e-2
            .single     1.0e-4
            .single     1.0e-8
            .single     1.0e-16
            .single     1.0e-32
            
exp32Tbl:   .word       0
            .word       1
            .word       2
            .word       4
            .word       8
            .word       16
            .word       32


   
			
//////////////////////////////////////////////////////////////
//
// u24toStr:
//
//  Converts a 24-bit unsigned integer to a string containing
//  exactly 8 digits.
//
//  Inputs:
//      R0:     pointer to buffer to receive string.
//      R1-  	unsigned 24-bit integer to convert.
//
//  Outputs:
//      Buffer: receives the zero-terminated string.															 
//      R0:     points at zero-terminating byte in string.
//
//  Buffer must have at least 9 bytes allocated for it.
//
// This code is a bit simplified from the u32toStr function
// due to the fact that it always emits exactly 7 digits
// (never any leading zeros).            
//
// Utility macro (just to save typing):
	
    .macro  divmod dest, src
    udiv    \dest, \src, r2
    mov     r4, \dest
    mls     r4, r4, r2, \src
    orr     r4, r4, #'0'
    strb    r4, [r0], #-1
    .endm


    proc    u24ToStr, thumb, public
    
	push	{r0-r4, lr}

	add		r0, r0, #maxSPdigits // End of buffer
	mov		r4, #0				 // Emit zero byte
	strb	r4, [r0], #-1
	
    // Handle digits 0 to 6 here (LO digit
	// first): 
    
	mov		r2, #10			// Divide by 10
	divmod	r3, r1			// LO digit
	divmod	r1, r3
	divmod	r3, r1
	divmod	r1, r3
	divmod	r3, r1
	divmod	r1, r3    	
	divmod	r3, r1			// HO digit    	

	pop		{r0-r4, pc}
    endp    u24ToStr
         
	
		    
            
/////////////////////////////////////////////////////////
//
// SPDigits:
//
//  Used to convert a single-precision floating-point 
//  value in S0 to a string of digits.
//
// Inputs:
//      S0: Single-precision value to convert
//      R0: Pointer to buffer to receive chars
//
// Outputs:
//      R0: Still points at buffer
//      R1: Contains exponent of the number
//      R2: Contains sign (space or '-')
 
    proc    	SPDigits, thumb, public

	push		{r0, r3, r4, r5, r6, r7, r8, r9, r10, lr}
	vpush		{d0, d1}
		    
    // Save sign in R9 for the time being:
	
    mov     	r9, #' '        // Assume sign is +

#define fp1 s2 
    
	// S2 holds 1.0
	
	ldr		 	r1, =one
    vldr.f32 	fp1, [r1]
    
    // Special case for 0.0:
    
    vcmp.f32 	s0, #0.0
	vmrs		APSR_nzcv, FPSCR
    bne     	s0not0
    ldr     	r1, =0x30303030
    str     	r1, [r0]
    strh    	r1, [r0, #4]
    strb    	r1, [r0, #6]
    mov     	r1, #0          // Exponent = 0
    strb    	r1, [r0, #7 ]   // Also zero terminator
	mov			r2, #' '		// Sign is positive
    b 	    	spdDone
            
// If the number is non-zero, deal with it here. Note
// that the flags were set by comparing S0 to 0.0 earlier.

s0not0:     
	bge    	 fpIsPositive    // See if positive or negative
        
    // If negative, negate and change the sign
    // character to '-'
    
    vabs.f32    s0, s0
    mov     	r9, #'-'

// Get the number between 1.0 and <10.0 so we can figure out 
// what the exponent is.  Begin by checking to see if we have 
// a positive or negative exponent.

fpIsPositive:

    mov     	r1, #0         	// Initialize exponent
    vcmp.f32    s0, fp1
	vmrs		APSR_nzcv, FPSCR
    bge     	posExp
    
    // The value is between 0.0 and 1.0,
    // exclusive,  at this point.  That means this
    // number has a negative  exponent.  Multiply
    // the number by an appropriate power of ten
    // until we get it in the range 1 through 10.

    ldr     	r6, =exp32Tbl
    mov     	r7, #expCnt

// Search through the potNeg32 table until we find a power
// of ten that is less than the value in D0:

cmpNegExp:          
    subs    	r7, r7, #1
    blt     	test1       // Branch if r7 < 1
    
    ldr     	r4, =potNeg32
	add			r4, r4, r7, lsl #2
    vldr.f32   	s1, [r4]  	// S1=potNeg32[r7*4]
    vcmp.f32	s1, s0      // repeat while
	vmrs		APSR_nzcv, FPSCR
    ble     	cmpNegExp   // table <= value           
            
    // Eliminate the current exponent indexed by
    // r7 by multiplying by the corresponding
    // entry in potPos32.

    ldr     	r8, [r6, r7, lsl #2] 	// r8=exp32Tbl[r7*4]
    sub     	r1, r1, r8
	
    ldr     	r5, =potPos32
	add			r5, r5, r7, lsl #2 
    vldr.f32	s1, [r5]  				// S1=potPos32[r7*4]
    vmul.f32   	s0, s0, s1
    b       	cmpNegExp 
            
// If we get to this point, then we've indexed through
// all the elements in the potNeg32 and it's time to stop.
//
// If the remainder is *exactly* 1.0, then we can branch
// on to InRange1_10; otherwise, we still have to multiply
// by 10.0 because we've overshot the mark a bit.

test1:
	vcmp.f32	s0, fp1
	vmrs		APSR_nzcv, FPSCR
    beq     	inRange1_10
    
	ldr			r3, =ten
    vldr.f32   	s1, [r3]
    vmul.f32	s0, s0, s1
    sub     	r1, r1, #1      // Decrement exponent
    b       	inRange1_10
	
            
            
//  At this point, we've got a number that is one or greater.
//  Once again, our task is to get the value between 1.0 and <10.0.
            
posExp:
    mov     	r7, #expCnt
    ldr     	r6, =exp32Tbl
       
cmpPosExp:
	subs    	r7, r7, #1
    blt     	inRange1_10     		// if r7 < 1
    ldr     	r5, =potPos32
	add			r5, r5, r7, lsl #2 
    vldr.f32    s1, [r5]				// D1=potPos32[r7*4]
    vcmp.f32	s1, s0
	vmrs		APSR_nzcv, FPSCR
    bgt     	cmpPosExp
    
    ldr     	r8, [r6, r7, lsl #2] 	// r8=exp32Tbl[r7*4]            
    add     	r1, r1, r8
    vdiv.f32   	s0, s0, s1
    b       	cmpPosExp
            

// Okay, at this point the number is in the range 1 <= x < 10,
// Let's multiply it by 1e+6 to put the most significant digit
// into the 7th print position.  Then convert the result to
// a string and store away in memory.
            
inRange1_10:
	mov				r10, r1			// Preserve exponent
    vldr.f32  		s1, tenTo6
    vmul.f32   		s0, s0, s1
	vcvt.u32.f32	s1, s0
	vmov.f32		r1, s1
	
    // Convert the integer mantissa to a 
    // string of digits:
 
    bl      	u24ToStr
            
	mov			r2, r9	// Return sign in R2.
	mov			r1, r10 // Return exponent in R1.
	 
spdDone:   
	vpop		{d0, d1}	// S0-S3
	pop			{r0, r3, r4, r5, r6, r7, r8, r9, r10, pc}
    endp    	SPDigits
                      

