// e64ToStr.S
//
// Floating-point (double) to string conversion.
//
// Exponential output.
																																		  
            #include    "ao32aaLibPrivate.inc"

            .text


                    
              
              
//***********************************************************
//                                                           
// e64ToBuf-                                                   
//                                                           
// Converts a double-precision floating-point number to the       
// corresponding string of digits.  Note that this           
// function always emits the string using scientific                  
// notation, use the r64ToStr routine for decimal notation.   
//                                                           
// On Entry:                                                 
//                                                           
//    D0-	Double-precision value to convert.
//          Passed in D0                     
//                                                  
//    R0-   e64ToStr stores the resulting characters in  
//          this buffer.
//                                                  
//    R1-   Field width for the number (note that this   
//          is an *exact* field width, not a minimum     
//          field width).
//                                                  
//    R2-   Padding character if the number is smaller   
//          than the specified field width.
//                                                                   
//    R3-   Number of exponent digits (2 for single     
//          and 3 for double).
//
//    R4-   Maximum buffer size.
//
// On Exit:                                                  
//
//	  R0-	Points at the end of the buffer (at the zero-
//			terminating string).
//                                                           
//    		Buffer contains the newly formatted string.      
//    		If the formatted value does not fit in the    
//    		width specified, e64ToStr will store "#" 
//			characters into this string. 
//
//  carry:  clear if no error, set if error. 
//          If error, R0=
//
//              -3 if string overflow
//              -2 if bad width
//              -1 if value out of range 
//                                                           
//-----------------------------------------------------------
//                                                           
// Unlike the integer to string conversions, this routine    
// always right justifies the number in the specified        
// string.  Width must be a positive number, negative        
// values are illegal (actually, they are treated as         
// *really* big positive numbers which will always raise     
// a string overflow exception.                              
//                                                           
//***********************************************************

            proc        e64ToStr, thumb, public
			push		{r0, lr}
			bl			e64ToBuf
			bcs			returnError
			pop			{r0, pc}
			
returnError:
			add			sp, sp, #4	// Keep error code in R0
			pop			{pc}
			endp		e64ToStr
			
			proc		e64ToBuf, thumb, public
			push		{r1-r12, lr}
			vpush		{d0}
			sub			sp, sp, #40	// Local bufer storage			

#define     e2sWidth   r5
#define     e2sFill    r6
#define     e2sBuffer  r7
#define     e2sExpDigs r8
#define     e2sMaxLen  r9

#define     e2sExp     r10
#define     e2sMantSz  r11
#define     e2sSign    r12
            
            
            mov     e2sBuffer, 	r0
            mov     e2sWidth, 	r1
            mov     e2sFill, 	r2
            mov     e2sExpDigs, r3
            mov     e2sMaxLen, 	r4

            cmp     e2sWidth, e2sMaxLen
            bhs     strOvfl 

			mov		r0, #0
            strb    r0, [e2sBuffer, e2sWidth]    // 0-terminate str
    
// First, make sure the width isn't zero.
            
            cmp     e2sWidth, #0
            beq     valOutOfRng

// Just to be on the safe side, don't allow widths greater 
// than 1024:

			ldr		r0, =1024
            cmp     e2sWidth, r0
            bhi     badWidth

// Okay, do the conversion.

            mov		r0, sp			// Local buffer storage
            bl      FPDigits        // Convert D0 to digit str
            mov     e2sExp, r1      // Save away exponent result
            mov     e2sSign, r2     // Save mantissa sign char

// Verify that there is sufficient room for the mantissa's sign,
// the decimal point, two mantissa digits, the "E", 
// and the exponent's sign.  Also add in the number of digits
// required by the exponent (2 for single, 3 for double).
//
// -1.2e+00    :real4
// -1.2e+000   :real8
            
            add     r2, e2sExpDigs, #6    // Minimum number of posns
            cmp     r2, e2sWidth
            bls     goodWidth
            
// Output a sequence of "#...#" chars (to the specified width)
// if the width value is not large enough to hold the 
// conversion:

            mov     r2, e2sWidth
            mov     r0, #'#'
            mov     r1, e2sBuffer
fillPnd:    strb    r0, [r1]
            add     r1, r1, #1
            subs    r2, r2, #1
            bne     fillPnd
			mov		r0, #0
			strb	r0, [r1]
            b       exit_eToBuf


// Okay, the width is sufficient to hold the number, do the
// conversion and output the string here:

goodWidth:
            // Compute the # of mantissa digits to display,
            // not counting mantissa sign, decimal point,
            // "E", and exponent sign.
                       
            sub     e2sMantSz, e2sWidth, e2sExpDigs
            sub     e2sMantSz, e2sMantSz, #4     
                    
            
            // Round the number to the specified number of
            // print positions. (Note: since there are a
            // maximum of 16 significant digits, don't
            // bother with the rounding if the field width
            // is greater than 16 digits.)
            
            cmp     e2sMantSz, #maxDPdigits
            bhs     noNeedToRound
                    
            // To round the value to the number of
            // significant digits, go to the digit just
            // beyond the last one we are considering (ebx
            // currently contains the number of decimal
            // positions) and add 5 to that digit.
            // Propagate any overflow into the remaining
            // digit positions.
            
            add     r1, e2sMantSz, #1
			mov		r2, sp				// Point R2 at local buffer
            ldrb    r0, [r2, r1]        // Get least sig digit + 1.
            add     r0, r0, #5          // Round (e.g., +0.5 ).
            cmp     r0, #'9'
            bhi     whileDigGT9
            b       noNeedToRound
            
// Sneak this code in here, after a branch, so the
// loop below doesn't get broken up.

firstDigitInNumber:

            // If we get to this point, then we've hit the
            // first digit in the number.  So we've got to
            // shift all the characters down one position
            // in the string of bytes and put a "1" in the
            // first character position.
            
            ldr     r0, [r2, #8]
            str     r0, [r2, #9]
            ldr     r0, [r2] 
            str     r0, [r2, #1]
            
            mov     r0, #'1'        // Store '1' in 1st
            strb    r0, [r2]        // digit position
            
            // Bump exponent by one as the shift did
            // a divide by 10.
            
            add     e2sExp, e2sExp, #1        
            b       noNeedToRound

// Subtract out overflow and add the carry into the previous
// digit (unless we hit the first digit in the number).

whileDigGT9:
            sub     r0, r0, #10
            strb    r0, [r2, r1]
            subs    r1, r1, #1
            bmi     firstDigitInNumber      
              
            // Add in carry to previous digit:
                                                                  
            ldrb    r0, [r2, r1]
            add     r0, r0, #1
            strb    r0, [r2, r1]
            cmp     r0, #'9'        // Overflow if char > '9'
            bhi     whileDigGT9 
            
noNeedToRound:      
			mov		r2, sp			// Point at local storage

            
// Okay, emit the string at this point.  This is pretty easy
// since all we really need to do is copy data from the
// digits array and add an exponent (plus a few other simple chars).
            
            mov     r1, #0      // Count output mantissa digits.
            strb    e2sSign, [e2sBuffer], #1
            
// Output the first character and a following decimal point
// if there are more than two mantissa digits to output.

            ldrb    r0, [r2]
            strb    r0, [e2sBuffer], #1
            add     r1, r1, #1
            cmp     r1, e2sMantSz
            beq     noDecPt
            
            mov     r0, #'.'
            strb    r0, [e2sBuffer], #1
                                    
noDecPt:
            
// Output any remaining mantissa digits here.
// Note that if the caller requests the output of
// more than 16 digits, this routine will output zeros
// for the additional digits.
            
            b       whileX2ltMantSizeTest
            
whileX2ltMantSize:
            
            mov     r0, #'0'
            cmp     r1, #maxDPdigits
            bhs     justPut0

            ldrb    r0, [r2, r1]
                    
justPut0:
            strb    r0, [e2sBuffer], #1
            add     r1, r1, #1
            
whileX2ltMantSizeTest:

            cmp     r1, e2sMantSz
            blo     whileX2ltMantSize

// Output the exponent:
            
            mov     r0, #'e'
            strb    r0, [e2sBuffer], #1
            mov     r0, #'+'
            mov     r4, #'-'
            neg     r3, e2sExp

            cmp     e2sExp, #0
			it		lt
			movlt	r0, r4
			it		lt
			movlt	e2sExp, r3
                                            
            strb    r0, [e2sBuffer], #1
 
           
            mov     r1, e2sExp
            mov     r2, e2sExpDigs
            mov     r0, e2sBuffer
            bl      expToBuf
            bcs     error
                    
exit_eToBuf:
            adds	r0, r0, #0	// clc = no error
                        
returnE64:
            add		sp, sp, #40
			vpop	{d0}
			pop		{r1-r12, pc}
			

strOvfl:    mov     r0, #-3
            b       error

badWidth:   mov     r0, #-2
            b       error
    
valOutOfRng:
            mov     r0, #-1
error:
			cmp		r0, #-3		// stc = error
			add		sp, sp, #40
			vpop	{d0}
			pop		{r1-r12, pc}
            endp    e64ToBuf
                
