// r32ToStr.S
//
// Floating-point (single) to string conversion.
//
// Provides decimal output formats.
//
																																		  
            #include    "ao32aaLibPrivate.inc"

            .text


//***********************************************************
//                                                           
// r32ToStr-                                                 
//                                                           
// Converts a single-precision floating-point number to the       
// corresponding string of digits.  Note that this           
// function always emits the string using decimal            
// notation.  For scientific notation, use the e32ToStr      
// routine.                                                  
//                                                           
// On Entry:                                                 
//                                                           
// S0-		Single-precision value to convert.
//                                                           
// R0-	    r32ToStr stores the resulting characters in  
//          this buffer.
//
// R1-      Field width for the number (note that this   
//          is an *exact* field width, not a minimum         
//          field width).
//                                                    
// R2-      # of digits to display after the decimal pt.
//                                                    
// R3-      Padding character if the number is smaller   
//          than the specified field width.
//                                                                     
// R4-      Maximum string length.
//                                                                            
// On Exit:                                                  
//                                                           
// Buffer contains the newly formatted string.  If the       
// formatted value does not fit in the width specified,      
// r64ToStr will store "#" characters into this string.      
//                                                           
// Carry-    Clear if success, set if an exception occurs.                                                         
//           If width is larger than the maximum length of          
//           the string specified by buffer, this routine        
//           will return with the carry set.                                             
//                                                           
//***********************************************************

// r32ToStr will preserve R0:

			proc	r32ToStr, thumb, public
			push	{r0, lr}
			bl		r32ToBuf
			bcs		returnError
			pop		{r0, pc}
			
returnError:
			add		sp, sp, #4
			pop		{pc}
			endp	r32ToStr
			

// r32ToBuf will leave R0 pointing at the end of the string:

            proc    r32ToBuf, thumb, public
			push	{r1-r11, lr}	// Note: must keep stack
			vpush	{d0-d1}			//       8-byte aligned!
			sub		sp, sp, #40		// Local storage for fp digits

            // Use meaningful names for the non-volatile
            // registers that hold local/parameter values:
                        
            #define fpVal 		r6
            #define fWidth 		r7
            #define decDigits 	r8
            #define exponent 	r9
            #define outBuf 		r10
            #define maxLength 	r11
            #define fill 		[sp, #34]
            #define sign 		[sp, #35]

                        
            // Move parameter values to non-volatile 
            // storage:

            mov     maxLength,	r4           
            strb    r3, fill 
            mov     decDigits,	r2 
            mov     fWidth, 	r1 
            mov     outBuf, 	r0 
            
            
            // First, make sure the number will fit into
            // the  specified string.
            
            cmp     fWidth, maxLength 
            bhs     strOverflow
            
            //  If the width is zero, return an error:

            cmp     fWidth, #0
            beq     valOutOfRange

            // Okay, do the conversion.  Begin by
            // processing the mantissa digits.
                        
            mov		r0, sp 				// Buffer for SPdigits        
            bl      SPDigits            // Convert D0 to string.

            mov     exponent, r1        // Save away exponent result.
            strb    r2, sign            // Save mantissa sign char.

// Round the string of digits to the number of significant 
// digits we want to display for this number. Note that there 
// are a maximum of seven digits produced for a 24-bit value.

			cmp		r1, #0			// R1=abs(exponent)
			it		lt
			neglt	r1, r1

            cmp     r1, #maxSPdigits
            bls     dontForceWidthZero
            mov     r1, #0          // If the exponent is negative, or
                                    // too large, set width to 0.
dontForceWidthZero:
            add     r1, r1, decDigits // Compute rounding position
            cmp     r1, #maxSPdigits
            bhs     dontRound       // Don't bother if a big #.

            // To round the value to the number of
            // significant digits, go to the digit just
            // beyond the last one we are considering (eax
            // currently contains the number of decimal
            // positions) and add 5 to that digit.
            // Propagate any overflow into the remaining
            // digit positions.
                    
                    
            add     r1, r1, #1          // Index+1 of last sig digit.
            ldrb    r0, [sp, r1]        // Get that digit.

            add     r0, r0, #5          // Round (e.g., +0.5 ).
            cmp     r0, #'9'
            bls     dontRound

            mov     r0, #('0' + 10)     // Force to zero
whileDigitGT9:                          
            sub     r0, r0, #10         // Sub out overflow,
            strb    r0, [sp, r1]        // carry, into prev

            subs    r1, r1, #1          // digit (until 1st
            bmi     hitFirstDigit       //  digit in the #)
             
            ldrb    r0, [sp, r1]        // Increment previous
            add     r0, r0, #1          // digit.
            strb    r0, [sp, r1]                           

            cmp     r0, #'9'            // Overflow if > '9'
            bhi     whileDigitGT9
            b       dontRound
 
hitFirstDigit:

            // If we get to this point, then we've hit the
            // first digit in the number.  So we've got to
            // shift all the characters down one position
            // in the string of bytes and put a "1" in the
            // first character position.
            
            mov     r2, #maxSPdigits      // Max digits in value
repeatUntilX2eq0:
            
            ldrb    r0, [sp, r2]
            add     r2, r2, #1
            strb    r0, [sp, r2]
            subs    r2, r2, #2
            bne     repeatUntilX2eq0
            
            mov     r0, #'1'
            strb    r0, [sp, r2]
             
            add     exponent, exponent, #1 // Increment exponent because
                                           // we added a digit
                    
dontRound:
            // Handle positive and negative exponents separately.

            mov     r5, #0             // Index into output buf
            cmp     exponent, #0
            bge     positiveExponent

            // Negative exponents:
            // Handle values between 0 & 1.0 here (negative
            // exponents imply negative powers of ten).
            // 
            // Compute the number's width.  Since this
            // value is between 0 & 1, the width
            // calculation is easy: it's just the number of
            // decimal positions they've specified plus
            // three (since we need to allow room for a
            // leading "-0."). R2=number of digits to emit
            // after "."

            mov     r4, #4            
            add     r2, decDigits, #3
            cmp     r2, r4
			it		lo
			movlo	r2, r4		// If X2<X4, X2=X4
		
            cmp     r2, fWidth
            bhi     widthTooBig 
            
            // This number will fit in the specified field
            // width, so output any necessary leading pad
            // characters. R3=number of padding characters
            // to output.
            
            sub     r3, fWidth, r2
            b       testWhileX3ltWidth

            
        
whileX3ltWidth:
			ldrb	r0, fill
            strb    r0, [outBuf, r5]
            add     r5, r5, #1          // Index
            add     r2, r2, #1          // Digits processed
testWhileX3ltWidth:
            cmp     r2, fWidth
            blo     whileX3ltWidth
                            
            // Output " 0." or "-0.", depending on 
            // the sign of the number.
            
			ldrb	r0, sign
            strb    r0, [outBuf, r5]
            add     r5, r5, #1
            mov     r0, #'0'
            strb    r0, [outBuf, r5]
            add     r5, r5, #1
            mov     r0, #'.'            
            strb    r0, [outBuf, r5]
            add     r5, r5, #1
            add     r3, r3, #3        
            
            // Now output the digits after the decimal point:

            mov     r2, #0              // Count the digits here
			mov		r1, sp				// Point at local storage
                                    
// If the exponent is currently negative, or if
// we've output more than 7 significant digits,
// just output a zero character.
            
repeatUntilX3geWidth:
            mov     r0, #'0'
            adds    exponent, exponent, #1
            bmi     noMoreOutput

            
            cmp     r2, #maxSPdigits
            bge     noMoreOutput
            
            ldrb    r0, [r1], #1
                    
noMoreOutput:
            strb    r0, [outBuf, r5]
            add     r5, r5, #1          // Index
            add     r2, r2, #1          // Digits processed
            add     r3, r3, #1          // Digit count
            cmp     r3, fWidth
            blo     repeatUntilX3geWidth
            b       r32BufDone


// If the number's actual width was bigger than the width
// specified by the caller, emit a sequence of '#' characters
// to denote the error.
                            
widthTooBig:    

            // The number won't fit in the specified field
            // width, so fill the string with the "#"
            // character to indicate an error.
                    
            mov     r2, fWidth
            mov     r0, #'#'
fillPound:  strb    r0, [outBuf, r5]
            add     r5, r5, #1          // Index
            subs    r2, r2, #1
            bne     fillPound
            b       r32BufDone

            
// Handle numbers with a positive exponent here.
//
// Compute # of print positions consumed by output string.
// This is given by:
//
//                   Exponent     // # of digits to left of "."
//           +       2            // Sign + 1's digit
//           +       decDigits    // Add in digits right of "."
//           +       1            // If there is a decimal point
            
positiveExponent:

            mov     r3, exponent    // Digits to left of "."
            add     r3, r3, #2      // sign posn.
            cmp     decDigits, #0   // See if any fractional
            beq     decPtsIs0       // part

            add     r3, r3, decDigits // Digits to right of "."
            add     r3, r3, #1         // Make room for the "."
            
            // Make sure the result will fit in the
            // specified field width.
            
decPtsIs0:
            cmp     r3, fWidth
            bhi     widthTooBig
            beq     noFillChars
            
                    
            // If the actual number of print positions
            // is less than the specified field width,
            // output leading pad characters here.
            
            subs    r2, fWidth, r3
            beq     noFillChars
            
			ldrb	r0, fill
fillChars:  strb    r0, [outBuf, r5]
            add     r5, r5, #1
            subs    r2, r2, #1
            bne     fillChars
                    
noFillChars:
 
            // Output the sign character.
            
			ldrb	r2, sign
            strb    r2, [outBuf, r5]
            add     r5, r5, #1
                    
            // Okay, output the digits for the number here.
            
            mov     r2, #0              // Counts  # of output chars.
			mov		r1, sp				// Point at local storage            
            
            // Calculate the number of digits to output
            // before and after the decimal point.
            
            add     r3, decDigits, exponent
            add     r3, r3, #1          // Always one digit before "."
                    
// If we've output fewer than seven digits, go ahead
// and output the next digit.  Beyond seven digits,
// output zeros.

            
repeatUntilX3eq0:
            mov     r0, #'0'
            cmp     r2, #maxSPdigits
            bhs     putChar
            
            ldrb    r0, [r1], #1
putChar:
			strb    r0, [outBuf, r5]
            add     r5, r5, #1     
            
            // If the exponent decrements down to zero,
            // then output a decimal point.
            
            cmp     exponent, #0
            bne     noDecimalPt
            
            cmp     decDigits, #0
            beq     noDecimalPt
            
            mov     r0, #'.'
            strb    r0, [outBuf, r5]
            add     r5, r5, #1
                    
noDecimalPt:
            sub     exponent, exponent, #1  // Count down to "." output
            add     r2, r2, #1    			// # of digits thus far
            subs    r3, r3, #1    			// Total # of digits to output
            bne     repeatUntilX3eq0
                    

// Zero-terminate string and leave:
            
r32BufDone: 
			mov		lr, #0
			strb    lr, [outBuf, r5]
			
			// Note that the following instruction will always
			// clear the carry flag, denoting no error (adding
			// an offset and an address will not produce an
			// overflow).
			
			adds	r0, outBuf, r5	// Point R0 at end of string
			
			add		sp, sp, #40		// Deallocate local storage
			vpop	{d0-d1}
			pop		{r1-r11, pc}

// Drop down here if there was an error creating the string:

strOverflow:
            mov     r0, #-3 //String overflow
            b       ErrorExit

valOutOfRange:       
			mov     r0, #-1 		// Range error
ErrorExit:  adds    r0, r0, #-1		// stc=error
            
            // Change r0 on return:

			add		sp, sp, #40		// Deallocate local storage
			vpop	{d0-d1}
			pop		{r1-r11, pc}
            endp    r32ToBuf                   
              
