// bufToH32.S
//
// Hexadecimal string to 32-bit integer conversion.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2
	
////////////////////////////////////////////////////
//
// bufToH32
//
// Hexadecimal string to 32-bit integer conversion.
// This code allows underscores to appear within a
// hex string (but underscores cannot be the first
// character of a hex string). This code also
// allows an underscore at the end of the string;
// if you consider this to be an error, modify
// the code at label "delimiter" to test for an
// underscore in the previous character position.
//
// On entry:
//
//	R0-		Pointer to buffer holding string.
//
// On exit-
//
//	R0-		Points at the first character beyond
//			the converted string, if conversion
//			was successful.
//
//	R1-		Contains the converted value if the
//			conversion was successful.
//
//	Carry-	Clear if conversion was successful.
//			Set if unsuccessful. If unsuccessful
//			conversion, then R0 will point at
//			the character the conversion failed
//			on and R1 will contain one of the
//			following error codes:
//
//			-1: First (non-WS) character was not
//				a valid hexadecimal digit.
//
//			-2: Overflow (value did not fit into
//				32 bits).
//
//			-3: Last character was not a valid
//				delimiter character (this could
//				be considered a "soft" error).
//
//	This function will skip any leading whitespace
// (spaces or tabs) appearing at the beginning
// of the string.

// Std version: 17s (Pi 400)
		
	proc	bufToH32, thumb, public
	push	{r2-r4, lr}
	
	mov		r1, #0			// Initialize result
	ldr		r3, =0x10000000	// For overflow check
	
	// Skip any leading whitespace, comma, semicolon,
	// or colon characters in the string:
	
	sub		r0, r0, #1		// To counteract pre-inc, below
skipDelim:
	ldrb	r2, [r0, #1]!
	
	cmp		r2, #' '
	beq		skipDelim
	cmp		r2, #tab
	beq		skipDelim
	cmp		r2, #','
	beq		skipDelim
	cmp		r2, #':'
	beq		skipDelim
	cmp		r2, #';'
	beq		skipDelim
	
	// Verify that the first character is a valid
	// hexadecimal digit:
	
	cmp		r2, #'0'
	blo		badFirst
	cmp		r2, #'9'
	bls		decDigit
	
	orr		r2, #0x20		// UC->lc
	
	cmp		r2, #'a'
	blo		badFirst
	cmp		r2, #'f'
	bls		hexDigit
	
	// If the first digit is bad, return -1:
	
badFirst:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-1
	cmp		r1, r1				// Sets carry flag
	pop		{r2-r4, pc} 


overflow:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-2
	cmp		r1, r1				// Sets carry flag
	pop		{r2-r4, pc}
	
	// Process hexadecimal and decimal digits:
	
hexDigit:
	add		r2, r2, #9			// Convert 'A'..'F' to 0xA..0xF	
decDigit:
	and		r2, r2, #0xf		// Convert to 0x0..0xf
	cmp		r1,	r3				// Check for overflow
	bhs		overflow 	
	orr		r1, r2, r1, lsl #4	// Merge into result

nextChar:
	ldrb	r2, [r0, #1]!
	cmp		r2, #'0'
	blo		tryDelimiter
	cmp		r2, #'9'
	bls		decDigit
	orr		r4, r2, #0x20		// UC->lc
	cmp		r4, #'a'
	blo		tryDelimiter
	cmp		r4, #'f'
	bls		hexDigit
		
tryDelimiter:
	cmp		r2, #'_'
	beq		nextChar

	cmp		r2, #0
	beq		delimiter
	cmp		r2, #' '
	beq		delimiter
	cmp		r2, #tab
	beq		delimiter
	cmp		r2, #','
	beq		delimiter
	cmp		r2, #':'
	beq		delimiter
	cmp		r2, #';'
	beq		delimiter
	
	// If the first non-hex-digit character
	// is bad, return -3:
	
illegal:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-3
	cmp		r1, r1				// Sets carry flag
	pop		{r2-r4, pc}

	// Transfer here when encountering a legitimate
	// delimiter character (valid end of hex string).
	
delimiter:
	sub		r0, r0, #1			// Point at delimiter char
	adds	r0, r0, #0			// Clears carry flag
	pop		{r2-r4, pc}

	endp	bufToH32
	 
/////////
//
// Classify: 34s
	
//	proc	bufToH32, thumb, public
//	push	{r2-r6, lr}
//	
//	mov		r1, #0			// Initialize result
//	ldr		r3, =0x10000000	// For overflow check
//	ldr		r4, =classTbl	// For char classification
//	
//	// Skip any leading whitespace, comma, semicolon,
//	// or colon characters in the string:
//	
//	sub		r0, r0, #1		// To counteract pre-inc, below
//skipDelim:
//	ldrb	r2, [r0, #1]!
//	cmp		r2, #0			// Is a delimiter, but not
//	beq		badFirst		// legal a first char!
//	ldrb	r5, [r4, r2]	// Get classification
//	cmp		r5, #3			// Is it a delimiter?
//	beq		skipDelim
//	
//	// First non-delimiter character must
//	// be a valid hexadecimal digit:
//	
//	cmp		r5, #1
//	blo		decDigit		// Decimal digit
//	beq		hexDigit		// Was a hex digit
//			
//	// If the first digit is bad, return -1:
//	
//badFirst:
//	sub		r0, r0, #1			// Point R0 at bad char
//	mov		r1, #-1
//	cmp		r1, r1				// Sets carry flag
//	pop		{r2-r6, pc} 
//
//overflow:
//	sub		r0, r0, #1			// Point R0 at bad char
//	mov		r1, #-2
//	cmp		r1, r1				// Sets carry flag
//	pop		{r2-r6, pc}
//	
//	// Process hexadecimal and decimal digits:
//	
//hexDigit:
//	add		r2, r2, #9			// Convert 'A'..'F' to 0xA..0xF	
//decDigit:
//	and		r2, r2, #0xf		// Convert to 0x0..0xf
//	cmp		r1,	r3				// Check for overflow
//	bhs		overflow 	
//	orr		r1, r2, r1, lsl #4	// Merge into result
//
//nextChar:
//	ldrb	r2, [r0, #1]!
//	ldrb	r5, [r4, r2]
//	cmp		r5, #1
//	blo		decDigit
//	beq		hexDigit
//	cmp		r5, #3
//	blo		nextChar	// underscore = 2
//	beq		delimiter
//		
//	// If the first non-hex-digit character
//	// is bad, return -3:
//	
//illegal:
//	sub		r0, r0, #1			// Point R0 at bad char
//	mov		r1, #-3
//	cmp		r1, r1				// Sets carry flag
//	pop		{r2-r6, pc}
//
//	// Transfer here when encountering a legitimate
//	// delimiter character (valid end of hex string).
//	
//delimiter:
//	sub		r0, r0, #1			// Point at delimiter char
//	adds	r0, r0, #0			// Clears carry flag
//	pop		{r2-r6, pc}
//
//	endp	bufToH32
//
//	// Classify each character:
//	//
//	//	0-	decimal character
//	//	1-	hexadecimal character
//	//	2-	underscore
//	//	3-	delimiter character
//	//	4-	illegal character
//	
//classTbl:
//	.byte	3	// 0x00
//	.byte	4	// 0x01
//	.byte	4	// 0x02
//	.byte	4	// 0x03
//	.byte	4	// 0x04
//	.byte	4	// 0x05
//	.byte	4	// 0x06
//	.byte	4	// 0x07
//	.byte	4	// 0x08
//	.byte	3	// 0x09
//	.byte	3	// 0x0a
//	.byte	4	// 0x0b
//	.byte	4	// 0x0c
//	.byte	3	// 0x0d
//	.byte	4	// 0x0e
//	.byte	4	// 0x0f
//	.byte	4	// 0x10
//	.byte	4	// 0x11
//	.byte	4	// 0x12
//	.byte	4	// 0x13
//	.byte	4	// 0x14
//	.byte	4	// 0x15
//	.byte	4	// 0x16
//	.byte	4	// 0x17
//	.byte	4	// 0x18
//	.byte	4	// 0x19
//	.byte	4	// 0x1a
//	.byte	4	// 0x1b
//	.byte	4	// 0x1c
//	.byte	4	// 0x1d
//	.byte	4	// 0x1e
//	.byte	4	// 0x1f
//	.byte	3	// ' '
//	.byte	4	// '!'
//	.byte	4	// '"'
//	.byte	4	// '#'
//	.byte	4	// '$'
//	.byte	4	// '%'
//	.byte	4	// '&'
//	.byte	4	// '''
//	.byte	4	// '('
//	.byte	4	// ')'
//	.byte	4	// '*'
//	.byte	4	// '+'
//	.byte	3	// ','
//	.byte	4	// '-'
//	.byte	4	// '.'
//	.byte	4	// '/'
//	.byte	0	// '0'
//	.byte	0	// '1'
//	.byte	0	// '2'
//	.byte	0	// '3'
//	.byte	0	// '4'
//	.byte	0	// '5'
//	.byte	0	// '6'
//	.byte	0	// '7'
//	.byte	0	// '8'
//	.byte	0	// '9'
//	.byte	3	// ':'
//	.byte	3	// ';'
//	.byte	4	// '<'
//	.byte	4	// '='
//	.byte	4	// '>'
//	.byte	4	// '?'
//	.byte	4	// '@'
//	.byte	1	// 'A'
//	.byte	1	// 'B'
//	.byte	1	// 'C'
//	.byte	1	// 'D'
//	.byte	1	// 'E'
//	.byte	1	// 'F'
//	.byte	4	// 'G'
//	.byte	4	// 'H'
//	.byte	4	// 'I'
//	.byte	4	// 'J'
//	.byte	4	// 'K'
//	.byte	4	// 'L'
//	.byte	4	// 'M'
//	.byte	4	// 'N'
//	.byte	4	// 'O'
//	.byte	4	// 'P'
//	.byte	4	// 'Q'
//	.byte	4	// 'R'
//	.byte	4	// 'S'
//	.byte	4	// 'T'
//	.byte	4	// 'U'
//	.byte	4	// 'V'
//	.byte	4	// 'W'
//	.byte	4	// 'X'
//	.byte	4	// 'Y'
//	.byte	4	// 'Z'
//	.byte	4	// '['
//	.byte	4	// '\'
//	.byte	4	// ']'
//	.byte	4	// '^'
//	.byte	2	// '_'
//	.byte	4	// '`'
//	.byte	1	// 'a'
//	.byte	1	// 'b'
//	.byte	1	// 'c'
//	.byte	1	// 'd'
//	.byte	1	// 'e'
//	.byte	1	// 'f'
//	.byte	4	// 'g'
//	.byte	4	// 'h'
//	.byte	4	// 'i'
//	.byte	4	// 'j'
//	.byte	4	// 'k'
//	.byte	4	// 'l'
//	.byte	4	// 'm'
//	.byte	4	// 'n'
//	.byte	4	// 'o'
//	.byte	4	// 'p'
//	.byte	4	// 'q'
//	.byte	4	// 'r'
//	.byte	4	// 's'
//	.byte	4	// 't'
//	.byte	4	// 'u'
//	.byte	4	// 'v'
//	.byte	4	// 'w'
//	.byte	4	// 'x'
//	.byte	4	// 'y'
//	.byte	4	// 'z'
//	.byte	4	// '{'
//	.byte	4	// '|'
//	.byte	4	// '}'
//	.byte	4	// '~'
//	.byte	4	// ''
//	.byte	4	// 0x80
//	.byte	4	// 0x81
//	.byte	4	// 0x82
//	.byte	4	// 0x83
//	.byte	4	// 0x84
//	.byte	4	// 0x85
//	.byte	4	// 0x86
//	.byte	4	// 0x87
//	.byte	4	// 0x88
//	.byte	4	// 0x89
//	.byte	4	// 0x8a
//	.byte	4	// 0x8b
//	.byte	4	// 0x8c
//	.byte	4	// 0x8d
//	.byte	4	// 0x8e
//	.byte	4	// 0x8f
//	.byte	4	// 0x90
//	.byte	4	// 0x91
//	.byte	4	// 0x92
//	.byte	4	// 0x93
//	.byte	4	// 0x94
//	.byte	4	// 0x95
//	.byte	4	// 0x96
//	.byte	4	// 0x97
//	.byte	4	// 0x98
//	.byte	4	// 0x99
//	.byte	4	// 0x9a
//	.byte	4	// 0x9b
//	.byte	4	// 0x9c
//	.byte	4	// 0x9d
//	.byte	4	// 0x9e
//	.byte	4	// 0x9f
//	.byte	4	// 0xa0
//	.byte	4	// 0xa1
//	.byte	4	// 0xa2
//	.byte	4	// 0xa3
//	.byte	4	// 0xa4
//	.byte	4	// 0xa5
//	.byte	4	// 0xa6
//	.byte	4	// 0xa7
//	.byte	4	// 0xa8
//	.byte	4	// 0xa9
//	.byte	4	// 0xaa
//	.byte	4	// 0xab
//	.byte	4	// 0xac
//	.byte	4	// 0xad
//	.byte	4	// 0xae
//	.byte	4	// 0xaf
//	.byte	4	// 0xb0
//	.byte	4	// 0xb1
//	.byte	4	// 0xb2
//	.byte	4	// 0xb3
//	.byte	4	// 0xb4
//	.byte	4	// 0xb5
//	.byte	4	// 0xb6
//	.byte	4	// 0xb7
//	.byte	4	// 0xb8
//	.byte	4	// 0xb9
//	.byte	4	// 0xba
//	.byte	4	// 0xbb
//	.byte	4	// 0xbc
//	.byte	4	// 0xbd
//	.byte	4	// 0xbe
//	.byte	4	// 0xbf
//	.byte	4	// 0xc0
//	.byte	4	// 0xc1
//	.byte	4	// 0xc2
//	.byte	4	// 0xc3
//	.byte	4	// 0xc4
//	.byte	4	// 0xc5
//	.byte	4	// 0xc6
//	.byte	4	// 0xc7
//	.byte	4	// 0xc8
//	.byte	4	// 0xc9
//	.byte	4	// 0xca
//	.byte	4	// 0xcb
//	.byte	4	// 0xcc
//	.byte	4	// 0xcd
//	.byte	4	// 0xce
//	.byte	4	// 0xcf
//	.byte	4	// 0xd0
//	.byte	4	// 0xd1
//	.byte	4	// 0xd2
//	.byte	4	// 0xd3
//	.byte	4	// 0xd4
//	.byte	4	// 0xd5
//	.byte	4	// 0xd6
//	.byte	4	// 0xd7
//	.byte	4	// 0xd8
//	.byte	4	// 0xd9
//	.byte	4	// 0xda
//	.byte	4	// 0xdb
//	.byte	4	// 0xdc
//	.byte	4	// 0xdd
//	.byte	4	// 0xde
//	.byte	4	// 0xdf
//	.byte	4	// 0xe0
//	.byte	4	// 0xe1
//	.byte	4	// 0xe2
//	.byte	4	// 0xe3
//	.byte	4	// 0xe4
//	.byte	4	// 0xe5
//	.byte	4	// 0xe6
//	.byte	4	// 0xe7
//	.byte	4	// 0xe8
//	.byte	4	// 0xe9
//	.byte	4	// 0xea
//	.byte	4	// 0xeb
//	.byte	4	// 0xec
//	.byte	4	// 0xed
//	.byte	4	// 0xee
//	.byte	4	// 0xef
//	.byte	4	// 0xf0
//	.byte	4	// 0xf1
//	.byte	4	// 0xf2
//	.byte	4	// 0xf3
//	.byte	4	// 0xf4
//	.byte	4	// 0xf5
//	.byte	4	// 0xf6
//	.byte	4	// 0xf7
//	.byte	4	// 0xf8
//	.byte	4	// 0xf9
//	.byte	4	// 0xfa
//	.byte	4	// 0xfb
//	.byte	4	// 0xfc
//	.byte	4	// 0xfd
//	.byte	4	// 0xfe
//	.byte	4	// 0xff
//	 
//	
// Jmp Table version: 26s
//
//	proc	bufToH32, thumb, public
//	push	{r2-r4, lr}
//	
//	mov		r1, #0			// Initialize result
//	ldr		r3, =0x10000000	// For overflow check
//	
//	// Skip any leading whitespace, comma, semicolon,
//	// or colon characters in the string:
//	
//	sub		r0, r0, #1		// To counteract pre-inc, below
//skipWS:
//	ldrb	r2, [r0, #1]!
//	
//	cmp		r2, #' '
//	beq		skipWS
//	cmp		r2, #tab
//	beq		skipWS
//	cmp		r2, #','
//	beq		skipWS
//	cmp		r2, #':'
//	beq		skipWS
//	cmp		r2, #';'
//	beq		skipWS
//	
//	// Verify that the first character is a valid
//	// hexadecimal digit:
//	
//	cmp		r2, #'0'
//	blo		badFirst
//	cmp		r2, #'9'
//	bls		decDigit
//	cmp		r2, #'A'
//	blo		badFirst
//	cmp		r2, #'F'
//	bls		hexDigit
//	cmp		r2, #'a'
//	blo		badFirst
//	cmp		r2, #'f'
//	bls		hexDigit
//	
//	// If the first digit is bad, return -1:
//	
//badFirst:
//	sub		r0, r0, #1			// Point R0 at bad char
//	mov		r1, #-1
//	cmp		r1, r1				// Sets carry flag
//	pop		{r2-r4, pc} 
//
//// Switch statement.
//// Switches on the ASCII character code in R2.
//// Branches to code that will handle that particular
//// character.
//
//convertLoop:
//	ldrb	r2, [r0, #1]!		// Get next character
//	tbh		[pc, r2, lsl #1]	// Switch on ASCII code
//hxTbl:
//	.hword	(delimiter-hxTbl)/2	// 0x00
//	.hword	(illegal-hxTbl)/2	// 0x01
//	.hword	(illegal-hxTbl)/2	// 0x02
//	.hword	(illegal-hxTbl)/2	// 0x03
//	.hword	(illegal-hxTbl)/2	// 0x04
//	.hword	(illegal-hxTbl)/2	// 0x05
//	.hword	(illegal-hxTbl)/2	// 0x06
//	.hword	(illegal-hxTbl)/2	// 0x07
//	.hword	(illegal-hxTbl)/2	// 0x08
//	.hword	(delimiter-hxTbl)/2	// 0x09
//	.hword	(delimiter-hxTbl)/2	// 0x0a
//	.hword	(illegal-hxTbl)/2	// 0x0b
//	.hword	(illegal-hxTbl)/2	// 0x0c
//	.hword	(delimiter-hxTbl)/2	// 0x0d
//	.hword	(illegal-hxTbl)/2	// 0x0e
//	.hword	(illegal-hxTbl)/2	// 0x0f
//	.hword	(illegal-hxTbl)/2	// 0x10
//	.hword	(illegal-hxTbl)/2	// 0x11
//	.hword	(illegal-hxTbl)/2	// 0x12
//	.hword	(illegal-hxTbl)/2	// 0x13
//	.hword	(illegal-hxTbl)/2	// 0x14
//	.hword	(illegal-hxTbl)/2	// 0x15
//	.hword	(illegal-hxTbl)/2	// 0x16
//	.hword	(illegal-hxTbl)/2	// 0x17
//	.hword	(illegal-hxTbl)/2	// 0x18
//	.hword	(illegal-hxTbl)/2	// 0x19
//	.hword	(illegal-hxTbl)/2	// 0x1a
//	.hword	(illegal-hxTbl)/2	// 0x1b
//	.hword	(illegal-hxTbl)/2	// 0x1c
//	.hword	(illegal-hxTbl)/2	// 0x1d
//	.hword	(illegal-hxTbl)/2	// 0x1e
//	.hword	(illegal-hxTbl)/2	// 0x1f
//	.hword	(delimiter-hxTbl)/2	// ' '
//	.hword	(illegal-hxTbl)/2	// '!'
//	.hword	(illegal-hxTbl)/2	// '"'
//	.hword	(illegal-hxTbl)/2	// '#'
//	.hword	(illegal-hxTbl)/2	// '$'
//	.hword	(illegal-hxTbl)/2	// '%'
//	.hword	(illegal-hxTbl)/2	// '&'
//	.hword	(illegal-hxTbl)/2	// '''
//	.hword	(illegal-hxTbl)/2	// '('
//	.hword	(illegal-hxTbl)/2	// ')'
//	.hword	(illegal-hxTbl)/2	// '*'
//	.hword	(illegal-hxTbl)/2	// '+'
//	.hword	(delimiter-hxTbl)/2	// ','
//	.hword	(illegal-hxTbl)/2	// '-'
//	.hword	(illegal-hxTbl)/2	// '.'
//	.hword	(illegal-hxTbl)/2	// '/'
//	.hword	(decDigit-hxTbl)/2	// '0'
//	.hword	(decDigit-hxTbl)/2	// '1'
//	.hword	(decDigit-hxTbl)/2	// '2'
//	.hword	(decDigit-hxTbl)/2	// '3'
//	.hword	(decDigit-hxTbl)/2	// '4'
//	.hword	(decDigit-hxTbl)/2	// '5'
//	.hword	(decDigit-hxTbl)/2	// '6'
//	.hword	(decDigit-hxTbl)/2	// '7'
//	.hword	(decDigit-hxTbl)/2	// '8'
//	.hword	(decDigit-hxTbl)/2	// '9'
//	.hword	(delimiter-hxTbl)/2	// ':'
//	.hword	(delimiter-hxTbl)/2	// ';'
//	.hword	(illegal-hxTbl)/2	// '<'
//	.hword	(illegal-hxTbl)/2	// '='
//	.hword	(illegal-hxTbl)/2	// '>'
//	.hword	(illegal-hxTbl)/2	// '?'
//	.hword	(illegal-hxTbl)/2	// '@'
//	.hword	(hexDigit-hxTbl)/2	// 'A'
//	.hword	(hexDigit-hxTbl)/2	// 'B'
//	.hword	(hexDigit-hxTbl)/2	// 'C'
//	.hword	(hexDigit-hxTbl)/2	// 'D'
//	.hword	(hexDigit-hxTbl)/2	// 'E'
//	.hword	(hexDigit-hxTbl)/2	// 'F'
//	.hword	(illegal-hxTbl)/2	// 'G'
//	.hword	(illegal-hxTbl)/2	// 'H'
//	.hword	(illegal-hxTbl)/2	// 'I'
//	.hword	(illegal-hxTbl)/2	// 'J'
//	.hword	(illegal-hxTbl)/2	// 'K'
//	.hword	(illegal-hxTbl)/2	// 'L'
//	.hword	(illegal-hxTbl)/2	// 'M'
//	.hword	(illegal-hxTbl)/2	// 'N'
//	.hword	(illegal-hxTbl)/2	// 'O'
//	.hword	(illegal-hxTbl)/2	// 'P'
//	.hword	(illegal-hxTbl)/2	// 'Q'
//	.hword	(illegal-hxTbl)/2	// 'R'
//	.hword	(illegal-hxTbl)/2	// 'S'
//	.hword	(illegal-hxTbl)/2	// 'T'
//	.hword	(illegal-hxTbl)/2	// 'U'
//	.hword	(illegal-hxTbl)/2	// 'V'
//	.hword	(illegal-hxTbl)/2	// 'W'
//	.hword	(illegal-hxTbl)/2	// 'X'
//	.hword	(illegal-hxTbl)/2	// 'Y'
//	.hword	(illegal-hxTbl)/2	// 'Z'
//	.hword	(illegal-hxTbl)/2	// '['
//	.hword	(illegal-hxTbl)/2	// '\'
//	.hword	(illegal-hxTbl)/2	// ']'
//	.hword	(illegal-hxTbl)/2	// '^'
//	.hword	(underscore-hxTbl)/2	// '_'
//	.hword	(illegal-hxTbl)/2	// '`'
//	.hword	(hexDigit-hxTbl)/2	// 'a'
//	.hword	(hexDigit-hxTbl)/2	// 'b'
//	.hword	(hexDigit-hxTbl)/2	// 'c'
//	.hword	(hexDigit-hxTbl)/2	// 'd'
//	.hword	(hexDigit-hxTbl)/2	// 'e'
//	.hword	(hexDigit-hxTbl)/2	// 'f'
//	.hword	(illegal-hxTbl)/2	// 'g'
//	.hword	(illegal-hxTbl)/2	// 'h'
//	.hword	(illegal-hxTbl)/2	// 'i'
//	.hword	(illegal-hxTbl)/2	// 'j'
//	.hword	(illegal-hxTbl)/2	// 'k'
//	.hword	(illegal-hxTbl)/2	// 'l'
//	.hword	(illegal-hxTbl)/2	// 'm'
//	.hword	(illegal-hxTbl)/2	// 'n'
//	.hword	(illegal-hxTbl)/2	// 'o'
//	.hword	(illegal-hxTbl)/2	// 'p'
//	.hword	(illegal-hxTbl)/2	// 'q'
//	.hword	(illegal-hxTbl)/2	// 'r'
//	.hword	(illegal-hxTbl)/2	// 's'
//	.hword	(illegal-hxTbl)/2	// 't'
//	.hword	(illegal-hxTbl)/2	// 'u'
//	.hword	(illegal-hxTbl)/2	// 'v'
//	.hword	(illegal-hxTbl)/2	// 'w'
//	.hword	(illegal-hxTbl)/2	// 'x'
//	.hword	(illegal-hxTbl)/2	// 'y'
//	.hword	(illegal-hxTbl)/2	// 'z'
//	.hword	(illegal-hxTbl)/2	// '{'
//	.hword	(illegal-hxTbl)/2	// '|'
//	.hword	(illegal-hxTbl)/2	// '}'
//	.hword	(illegal-hxTbl)/2	// '~'
//	.hword	(illegal-hxTbl)/2	// ''
//	.hword	(illegal-hxTbl)/2	// 0x80
//	.hword	(illegal-hxTbl)/2	// 0x81
//	.hword	(illegal-hxTbl)/2	// 0x82
//	.hword	(illegal-hxTbl)/2	// 0x83
//	.hword	(illegal-hxTbl)/2	// 0x84
//	.hword	(illegal-hxTbl)/2	// 0x85
//	.hword	(illegal-hxTbl)/2	// 0x86
//	.hword	(illegal-hxTbl)/2	// 0x87
//	.hword	(illegal-hxTbl)/2	// 0x88
//	.hword	(illegal-hxTbl)/2	// 0x89
//	.hword	(illegal-hxTbl)/2	// 0x8a
//	.hword	(illegal-hxTbl)/2	// 0x8b
//	.hword	(illegal-hxTbl)/2	// 0x8c
//	.hword	(illegal-hxTbl)/2	// 0x8d
//	.hword	(illegal-hxTbl)/2	// 0x8e
//	.hword	(illegal-hxTbl)/2	// 0x8f
//	.hword	(illegal-hxTbl)/2	// 0x90
//	.hword	(illegal-hxTbl)/2	// 0x91
//	.hword	(illegal-hxTbl)/2	// 0x92
//	.hword	(illegal-hxTbl)/2	// 0x93
//	.hword	(illegal-hxTbl)/2	// 0x94
//	.hword	(illegal-hxTbl)/2	// 0x95
//	.hword	(illegal-hxTbl)/2	// 0x96
//	.hword	(illegal-hxTbl)/2	// 0x97
//	.hword	(illegal-hxTbl)/2	// 0x98
//	.hword	(illegal-hxTbl)/2	// 0x99
//	.hword	(illegal-hxTbl)/2	// 0x9a
//	.hword	(illegal-hxTbl)/2	// 0x9b
//	.hword	(illegal-hxTbl)/2	// 0x9c
//	.hword	(illegal-hxTbl)/2	// 0x9d
//	.hword	(illegal-hxTbl)/2	// 0x9e
//	.hword	(illegal-hxTbl)/2	// 0x9f
//	.hword	(illegal-hxTbl)/2	// 0xa0
//	.hword	(illegal-hxTbl)/2	// 0xa1
//	.hword	(illegal-hxTbl)/2	// 0xa2
//	.hword	(illegal-hxTbl)/2	// 0xa3
//	.hword	(illegal-hxTbl)/2	// 0xa4
//	.hword	(illegal-hxTbl)/2	// 0xa5
//	.hword	(illegal-hxTbl)/2	// 0xa6
//	.hword	(illegal-hxTbl)/2	// 0xa7
//	.hword	(illegal-hxTbl)/2	// 0xa8
//	.hword	(illegal-hxTbl)/2	// 0xa9
//	.hword	(illegal-hxTbl)/2	// 0xaa
//	.hword	(illegal-hxTbl)/2	// 0xab
//	.hword	(illegal-hxTbl)/2	// 0xac
//	.hword	(illegal-hxTbl)/2	// 0xad
//	.hword	(illegal-hxTbl)/2	// 0xae
//	.hword	(illegal-hxTbl)/2	// 0xaf
//	.hword	(illegal-hxTbl)/2	// 0xb0
//	.hword	(illegal-hxTbl)/2	// 0xb1
//	.hword	(illegal-hxTbl)/2	// 0xb2
//	.hword	(illegal-hxTbl)/2	// 0xb3
//	.hword	(illegal-hxTbl)/2	// 0xb4
//	.hword	(illegal-hxTbl)/2	// 0xb5
//	.hword	(illegal-hxTbl)/2	// 0xb6
//	.hword	(illegal-hxTbl)/2	// 0xb7
//	.hword	(illegal-hxTbl)/2	// 0xb8
//	.hword	(illegal-hxTbl)/2	// 0xb9
//	.hword	(illegal-hxTbl)/2	// 0xba
//	.hword	(illegal-hxTbl)/2	// 0xbb
//	.hword	(illegal-hxTbl)/2	// 0xbc
//	.hword	(illegal-hxTbl)/2	// 0xbd
//	.hword	(illegal-hxTbl)/2	// 0xbe
//	.hword	(illegal-hxTbl)/2	// 0xbf
//	.hword	(illegal-hxTbl)/2	// 0xc0
//	.hword	(illegal-hxTbl)/2	// 0xc1
//	.hword	(illegal-hxTbl)/2	// 0xc2
//	.hword	(illegal-hxTbl)/2	// 0xc3
//	.hword	(illegal-hxTbl)/2	// 0xc4
//	.hword	(illegal-hxTbl)/2	// 0xc5
//	.hword	(illegal-hxTbl)/2	// 0xc6
//	.hword	(illegal-hxTbl)/2	// 0xc7
//	.hword	(illegal-hxTbl)/2	// 0xc8
//	.hword	(illegal-hxTbl)/2	// 0xc9
//	.hword	(illegal-hxTbl)/2	// 0xca
//	.hword	(illegal-hxTbl)/2	// 0xcb
//	.hword	(illegal-hxTbl)/2	// 0xcc
//	.hword	(illegal-hxTbl)/2	// 0xcd
//	.hword	(illegal-hxTbl)/2	// 0xce
//	.hword	(illegal-hxTbl)/2	// 0xcf
//	.hword	(illegal-hxTbl)/2	// 0xd0
//	.hword	(illegal-hxTbl)/2	// 0xd1
//	.hword	(illegal-hxTbl)/2	// 0xd2
//	.hword	(illegal-hxTbl)/2	// 0xd3
//	.hword	(illegal-hxTbl)/2	// 0xd4
//	.hword	(illegal-hxTbl)/2	// 0xd5
//	.hword	(illegal-hxTbl)/2	// 0xd6
//	.hword	(illegal-hxTbl)/2	// 0xd7
//	.hword	(illegal-hxTbl)/2	// 0xd8
//	.hword	(illegal-hxTbl)/2	// 0xd9
//	.hword	(illegal-hxTbl)/2	// 0xda
//	.hword	(illegal-hxTbl)/2	// 0xdb
//	.hword	(illegal-hxTbl)/2	// 0xdc
//	.hword	(illegal-hxTbl)/2	// 0xdd
//	.hword	(illegal-hxTbl)/2	// 0xde
//	.hword	(illegal-hxTbl)/2	// 0xdf
//	.hword	(illegal-hxTbl)/2	// 0xe0
//	.hword	(illegal-hxTbl)/2	// 0xe1
//	.hword	(illegal-hxTbl)/2	// 0xe2
//	.hword	(illegal-hxTbl)/2	// 0xe3
//	.hword	(illegal-hxTbl)/2	// 0xe4
//	.hword	(illegal-hxTbl)/2	// 0xe5
//	.hword	(illegal-hxTbl)/2	// 0xe6
//	.hword	(illegal-hxTbl)/2	// 0xe7
//	.hword	(illegal-hxTbl)/2	// 0xe8
//	.hword	(illegal-hxTbl)/2	// 0xe9
//	.hword	(illegal-hxTbl)/2	// 0xea
//	.hword	(illegal-hxTbl)/2	// 0xeb
//	.hword	(illegal-hxTbl)/2	// 0xec
//	.hword	(illegal-hxTbl)/2	// 0xed
//	.hword	(illegal-hxTbl)/2	// 0xee
//	.hword	(illegal-hxTbl)/2	// 0xef
//	.hword	(illegal-hxTbl)/2	// 0xf0
//	.hword	(illegal-hxTbl)/2	// 0xf1
//	.hword	(illegal-hxTbl)/2	// 0xf2
//	.hword	(illegal-hxTbl)/2	// 0xf3
//	.hword	(illegal-hxTbl)/2	// 0xf4
//	.hword	(illegal-hxTbl)/2	// 0xf5
//	.hword	(illegal-hxTbl)/2	// 0xf6
//	.hword	(illegal-hxTbl)/2	// 0xf7
//	.hword	(illegal-hxTbl)/2	// 0xf8
//	.hword	(illegal-hxTbl)/2	// 0xf9
//	.hword	(illegal-hxTbl)/2	// 0xfa
//	.hword	(illegal-hxTbl)/2	// 0xfb
//	.hword	(illegal-hxTbl)/2	// 0xfc
//	.hword	(illegal-hxTbl)/2	// 0xfd
//	.hword	(illegal-hxTbl)/2	// 0xfe
//	.hword	(illegal-hxTbl)/2	// 0xff
//
//	// Note: TBH only branches forward, so
//	// this code must be below the jump table.
//
//	// Underscore just skips over the character:
//	
//underscore:
//	b		convertLoop
//	
//	
//	// If an overflow occurs, bail here:
//	
//overflow:
//	sub		r0, r0, #1			// Point R0 at bad char
//	mov		r1, #-2
//	cmp		r1, r1				// Sets carry flag
//	pop		{r2-r4, pc}
//
//	// If the first non-hex-digit character
//	// is bad, return -3:
//	
//illegal:
//	sub		r0, r0, #1			// Point R0 at bad char
//	mov		r1, #-3
//	cmp		r1, r1				// Sets carry flag
//	pop		{r2-r4, pc}
//
//	// Transfer here when encountering a legitimate
//	// delimiter character (valid end of hex string).
//	
//delimiter:
//	sub		r0, r0, #1			// Point at delimiter char
//	adds	r0, r0, #0			// Clears carry flag
//	pop		{r2-r4, pc}
//	
//	
//	// Transfer here if the character is
//	// A to F or a to f.
//	
//hexDigit:
//	cmp		r1,	r3				// Check for overflow
//	bhs		overflow 	
//	and		r2, r2, #0xf		// Convert to 1..6
//	add		r2, r2, #9			// Convert to 0xA..0xF
//	orr		r1, r2, r1, lsl #4	// Merge into result
//	b		convertLoop
//
//	// Transfer here if the character is
//	// 0 to 9.
//	
//decDigit:
//	cmp		r1,	r3				// Check for overflow
//	bhs		overflow 	
//	and		r2, r2, #0xf		// Convert to 0..9
//	orr		r1, r2, r1, lsl #4	// Merge into result
//	b		convertLoop
//	endp	bufToH32