// bufToH64.S
//
// Hexadecimal string to 64-bit integer conversion.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2
	
////////////////////////////////////////////////////
//
// bufToH64
//
// Hexadecimal string to 64-bit integer conversion.
// This code allows underscores to appear within a
// hex string (but underscores cannot be the first
// character of a hex string). This code also
// allows an underscore at the end of the string;
// if you consider this to be an error, modify
// the code at label "delimiter" to test for an
// underscore in the previous character position.
//
// On entry:
//
//	R0-		Pointer to buffer holding string.
//
// On exit-
//
//	R0-		Points at the first character beyond
//			the converted string, if conversion
//			was successful.
//
//	R2:R1-	Contains the converted value if the
//			conversion was successful.
//
//	Carry-	Clear if conversion was successful.
//			Set if unsuccessful. If unsuccessful
//			conversion, then R0 will point at
//			the character the conversion failed
//			on and R1 will contain one of the
//			following error codes:
//
//			-1: First (non-WS) character was not
//				a valid hexadecimal digit.
//
//			-2: Overflow (value did not fit into
//				32 bits).
//
//			-3: Last character was not a valid
//				delimiter character (this could
//				be considered a "soft" error).
//
//	This function will skip any leading whitespace
// (spaces or tabs) appearing at the beginning
// of the string.

		
	proc	bufToH64, thumb, public
	push	{r3-r5, lr}
	
	mov		r1, #0			// Initialize result
	mov		r2, #0
	ldr		r4, =0x10000000	// For overflow check
	
	// Skip any leading whitespace, comma, semicolon,
	// or colon characters in the string:
	
	sub		r0, r0, #1		// To counteract pre-inc, below
skipDelim:
	ldrb	r3, [r0, #1]!
	
	cmp		r3, #' '
	beq		skipDelim
	cmp		r3, #tab
	beq		skipDelim
	cmp		r3, #','
	beq		skipDelim
	cmp		r3, #':'
	beq		skipDelim
	cmp		r3, #';'
	beq		skipDelim
	
	// Verify that the first character is a valid
	// hexadecimal digit:
	
	cmp		r3, #'0'
	blo		badFirst
	cmp		r3, #'9'
	bls		decDigit
	
	orr		r3, #0x20		// UC->lc
	
	cmp		r3, #'a'
	blo		badFirst
	cmp		r3, #'f'
	bls		hexDigit
	
	// If the first digit is bad, return -1:
	
badFirst:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-1
	cmp		r1, r1				// Sets carry flag
	pop		{r3-r5, pc} 


overflow:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-2
	cmp		r1, r1				// Sets carry flag
	pop		{r3-r5, pc}
	
	// Process hexadecimal and decimal digits:
	
hexDigit:
	add		r3, r3, #9			// Convert 'A'..'F' to 0xA..0xF	
decDigit:
	cmp		r2,	r4				// Check for overflow
	bhs		overflow
	
	lsl		r2, r2, #4			// Shift upper 4 bits
	ror		r5, r1, #28			// from R1 into LO 4 bits
	and		r5, r5, #0xf		// of R2
	orr 	r2, r2, r5
	 	
	and		r3, r3, #0xf		// Convert to 0x0..0xf
	orr		r1, r3, r1, lsl #4	// Merge char into result

nextChar:
	ldrb	r3, [r0, #1]!
	cmp		r3, #'0'
	blo		tryDelimiter
	cmp		r3, #'9'
	bls		decDigit
	orr		r5, r3, #0x20		// UC->lc
	cmp		r5, #'a'
	blo		tryDelimiter
	cmp		r5, #'f'
	bls		hexDigit
		
tryDelimiter:
	cmp		r3, #'_'
	beq		nextChar

	cmp		r3, #0
	beq		delimiter
	cmp		r3, #' '
	beq		delimiter
	cmp		r3, #tab
	beq		delimiter
	cmp		r3, #','
	beq		delimiter
	cmp		r3, #':'
	beq		delimiter
	cmp		r3, #';'
	beq		delimiter
	
	// If the first non-hex-digit character
	// is bad, return -3:
	
illegal:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-3
	cmp		r1, r1				// Sets carry flag
	pop		{r3-r5, pc}

	// Transfer here when encountering a legitimate
	// delimiter character (valid end of hex string).
	
delimiter:
	sub		r0, r0, #1			// Point at delimiter char
	adds	r0, r0, #0			// Clears carry flag
	pop		{r3-r5, pc}
	endp	bufToH64
	 
