// bufToR64.S
//
// Real string to 64-bit floating-point 
// conversion.
//
// Handles decimal and exponential forms

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2
	.set			false, 0
	
// potPosl.
//
// Power of tens table (pot) used to quickly
// multiply or divide a floating-point value
// by powers of ten.
            
PotTbl:     .double     1.0e+128
            .double     1.0e+64
            .double     1.0e+32
            .double     1.0e+16
            .double     1.0e+8
            .double     1.0e+4
            .double     1.0e+2
            .double     1.0e+1
            .double     1.0e+0		// Dummy entry

	
//*********************************************************
//                                                         
// bufToR64-                                                   
//
// On entry:
//                                                         
//  R0: points at a string of characters that represent a   
//      floating point value.  
//
// On return:
//
//  D0: Converted result
//  R0: On return, R0 points at the first character this 
//      routine couldn't convert (if no error).
//
//  C:  Carry flag is clear if no error, set if error.
//      R0 is preserved if an error, R1 contains an
//      error code if an error occurs (else R1 is
//      preserved).          
//                                                         
//*********************************************************


            proc    bufToR64, thumb, public
			push	{r1-r9, lr}
			vpush	{d1}
            
            
// Defines to give registers more
// meaningful names:

#define mant    r8      //Mantissa value
#define mantHO	r9

#define sigDig  r2      //Significant digits
#define expAcc  r2      //Exponent accumulator
#define sign    r3      //Mantissa sign
#define fpExp   r4      //Exponent
#define expSign r5      //Exponent sign
#define ch      r6      //Current character
#define ten     r7      //The value 10
#define temp    r7      //Used to multiply by 10

            
    // Useful initialization:
    
    mov     fpExp, 	#0       //R4 Decimal exponent value
    mov     mant,  	#0       //R8 Mantissa value    
    mov     mantHO, #0       //R9 HO Mantissa value    
    mov     sign,  	#0       //R3 Assume non-negative
    
    // Initialize sigDig with 16, the number of
    // significant digits left to process.
    
    mov     sigDig, #16     //R2
                    
// Skip over any leading space, tab, comma, colon, or semicoon
// characters in the sequence:
            
	sub     r0, r0, #1      //Will inc'd in loop
whileWSLoop:
    ldrb    ch, [r0, #1]!   //R6
    cmp     ch, #' '
    beq     whileWSLoop
    cmp     ch, #tab
    beq     whileWSLoop
	cmp		ch, #','
	beq		whileWSLoop
	cmp		ch, #':'
	beq		whileWSLoop
	cmp		ch, #';'
	beq		whileWSLoop
    
    // Check for + or -
    
    cmp     ch, #'+'
    beq     skipSign
    
    cmp     ch, #'-'
	it		eq
	addeq	sign, sign, #1	//R3
    bne     noSign
            
skipSign:   
	ldrb    ch, [r0, #1]!   //Skip '-' or '+'
	
noSign: 
            
    // At this point we're beyond any leading sign
    // character. Therefore, the next character must
    // be a decimal digit or a decimal point.
    
    sub     ch, ch, #'0'    //Quick test for '0' to '9'
    cmp     ch, #9
    bls     scanDigits      //Branch if '0' to '9'

    cmp     ch, #'.'-'0'    //Check for '.'  
    bne     convError

    
    // If the first character is a decimal point,
    // then the second character needs to be a
    // decimal digit.
                    
    ldrb    ch, [r0, #1]!   //R6 Skip period
    cmp     ch, #'0'
    blo     convError
    cmp     ch, #'9'
    bhi     convError
    b       whileDigit2		// Do chars after '.' 

// Scan for digits at the beginning of the number:

scanDigits: 
	add     ch, ch, #'0'    //Restore character
	
whileADigit:
    sub     ch, ch, #'0'    //Quick way to test for
    cmp     ch, #10         // a range and convert
    bhs     notDigit        // to an integer

    // Ignore any leading zeros in the number
    // We have a leading '0' if the mantissa is 0
    // and the current character is '0'
    
    cmp     mant, #0        //Ignore leading zeros
	it		eq
	cmpeq	ch, #0
    beq     Beyond16
    
    // Each digit to the left of the decimal
    // point increases the number by an
    // additional power of ten.  Deal with that
    // here.

    add     fpExp, fpExp, #1     

    // Save all the significant digits, but ignore
    // any digits beyond the 16th digit.
    
    cmp     sigDig, #0		//R2
    beq     Beyond16
    
    // Count down the number of significant digits
    
    sub     sigDig, sigDig, #1
    
    // Multiply the accumulator (mant) by 10 and
    // add in the current digit. Note that ch
    // has already been converted to an integer.
	// Because R64 values have a 53-bit mantissa,
	// must do 64-bit arithmetic here.
	
    adds	mant, mant, mant		// mant * 2
	adc		mantHO, mantHO, mantHO	// (64 bits)
	
	mov		r1, mant				// Save mant * 2
	mov		temp, mantHO
	
    adds	mant, mant, mant		// mant * 4
	adc		mantHO, mantHO, mantHO	// (64 bits)
    adds	mant, mant, mant		// mant * 8
	adc		mantHO, mantHO, mantHO	// (64 bits)
	
	adds	mant, mant, r1			// mant * 10
	adc		mantHO, mantHO, temp
	
	add		mant, mant, ch			// Add in char.
	
	
	            
    // Because we multiplied the exponent by 10,
    // we need to undo the increment of fpExp.
    
    sub     fpExp, fpExp, #1                
                    
Beyond16:
	ldrb    ch, [r0, #1]!   //Get next char
    b       whileADigit
            
            
// If we encountered a non-digit character,
// check for a decimal point:
            
notDigit:
	add		ch, ch, #'0'   
    cmp     ch, #'.'		//See if the decimal point
    bne     whileDigit2
                        
// Okay, process any digits to the right of the decimal point.
// If this code falls through from the above, it skips the
// decimal point.
            
getNextChar:
	ldrb    ch, [r0, #1]!   //Get the next character
	
whileDigit2:
    sub     ch, ch, #'0'
    cmp     ch, #10
    bhs     noDigit2a
    
    // Ignore digits after the 16th signficant
    // digit, but dont count leading zeros
    // as signficant digits.

    cmp     mant, #0            //Ignore leading zeros
	bne		trySigDig
	cmp		ch, #0				//Is it a leading zero?
	beq		getNextChar
	b		notLeading0

trySigDig:
	cmp		sigDig, #0			// R2
	beq		getNextChar
          
    // Each digit to the right of the decimal point decreases
    // the number by an additional power of ten.  Deal with
    // that here.

notLeading0:
    sub     fpExp, fpExp, #1     

    // Count down the number of significant digits
    
    sub     sigDig, sigDig, #1

    // Multiply the accumulator (mant) by 10 and
    // add in the current digit. Note that ch
    // has already been converted to an integer.
    
    adds	mant, mant, mant		// mant * 2
	adc		mantHO, mantHO, mantHO	// (64 bits)
	
	mov		r1, mant				// Save mant * 2
	mov		temp, mantHO
	
    adds	mant, mant, mant		// mant * 4
	adc		mantHO, mantHO, mantHO	// (64 bits)
    adds	mant, mant, mant		// mant * 8
	adc		mantHO, mantHO, mantHO	// (64 bits)
	
	adds	mant, mant, r1			// mant * 10
	adc		mantHO, mantHO, temp
	
	add		mant, mant, ch			// Add in char.
    b       getNextChar

                    
// At this point, we've finished processing the mantissa.
// Now see if there is an exponent we need to deal with.

noDigit2a:
	add		ch, ch, #'0'
	
noDigit2:
    mov     expSign, #0     //R5 Initialize exp sign
    mov     expAcc, #0      //R2 Initialize exponent
    cmp     ch, #'e'
    beq     hasExponent
    cmp     ch, #'E'
    bne     noExponent
            
hasExponent:
    ldrb    ch, [r0, #1]!           // Skip the "E"
    cmp     ch, #'-'                //R6
	it		eq
    addeq   expSign, expSign, #1    //R5
    beq     doNextChar_2
    cmp     ch, #'+'
    bne     getExponent
            
doNextChar_2:
	ldrb    ch, [r0, #1]!   //Skip '+' or '-'
                    
            
// Okay, we're past the "E" and the optional sign at this
// point.  We must have at least one decimal digit.
            
getExponent:
    sub     ch, ch, #'0'    //R6
    cmp     ch, #10
    bhs     convError
    
    mov     expAcc, #0		// Compute exponent value in R2
ExpLoop:    
	ldrb    ch, [r0], #1
    sub     ch, ch, #'0'
    cmp     ch, #10
    bhs     ExpDone

	mov		ten, #10            
    mla     expAcc, expAcc, ten, ch    //R2, R7, R6
    b       ExpLoop

            
// If the exponent was negative, negate our computed result.
            
ExpDone:
    cmp     expSign, #false //R5
    beq     noNegExp
    
    neg     expAcc, expAcc  //R2
                    
noNegExp:

// Add in the computed decimal exponent with the exponent
// accumulator.
            
	add     fpExp, fpExp, expAcc    //R4, R2
            
noExponent:
                    
// verify that the exponent is between -324 and +308 (which
// is the maximum dynamic range for a 64-bit FP value).

    ldr     r6, =308        //Reuse ch here
    cmp     fpExp, r6
    bgt     voor            //Value out of range
    ldr     r6, =-324
    cmp     fpExp, r6
    blt     voor
            
// Now convert the mantissa value into a float.

	orrs	r1, mant, mantHO
	bne		notZero
	vmov	d0, r1, r1
	b		convSuccess

// Special case for 32-bit integer results:
			
notZero:
	cmp		mantHO, #0
	bne		mantIs64Bits
	
	// For the special case where the
	// mantissa is a 32-bit integer,
	// use the hardware instructions
	// to convert it to a double:
				
	vmov.f32		s0, mant	// R8
	vcvt.f64.u32	d0, s0
	b				convertedMant
	
	// If the mantissa doesn't fit into 32 bits, then
	// we have to do it 32 bits at a time:
		
mantIs64Bits:
	vmov.f32		s0, mant	// R8
	vcvt.f64.u32	d0, s0
	vmov.f32		s2, mantHO	// R9
	vcvt.f64.u32	d1, s2
	
	// At this point, d1 is 2**32 smaller than it
	// should be. Fix this by adding 32 to the exponent.
	
	vmov.f64		mant, mantHO, d1
	ldr				r1, =0x02000000		// 32 << 52
	add				mantHO, mantHO, r1
	vmov.f64		d1, mant, mantHO
	
	// Okay, produce the full integer value by adding D0
	// and D1:
	
	vadd.f64		d0, d0, d1
	
// Okay, we've got the mantissa into D0.  Now multiply 
// D0 by 10 raised to the value of the computed exponent
// (currently in fpExp).
//
// This code uses power of 10 tables to help make the 
// computation a little more accurate.
//
// We want to determine which power of ten is just less than the
// value of our exponent.  The powers of ten to check are
// 10**256, 10**128, 10**64, 10**32, etc.  A slick way to
// do this check is by shifting the bits in the exponent
// to the left.  Bit #8 is the 256 bit.  So if this bit is set,
// our exponent is >= 10**256.  If not, check the next bit down
// to see if our exponent >= 10**128, etc.

convertedMant:
    mov     	r2, -8  	//Initial index into power of ten table
    cmp     	fpExp, #0	//R4
    bge     	positiveExponent
    
    // Handle negative exponents here.
    
    neg     	fpExp, fpExp
    lsl     	fpExp, fpExp, #24 //Bits 0..7 -> 24..31
    ldr     	r7, =PotTbl
whileExpNE0:
    add     	r2, r2, #8          //Next index into PotTbl
    lsls    	fpExp, fpExp, #1	//Next bit from fpExpr
    bcc     	testExp0
	add			r1, r7, r2
    vldr     	d1, [r1]
    vdiv.f64    d0, d0, d1

testExp0:   
	cmp     	fpExp, #0
    bne     	whileExpNE0
    b       	doMantissaSign


// Handle positive exponents here.
                    
positiveExponent:
    ldr     	r7, =PotTbl
    lsl     	fpExp, fpExp, #24       // Bits 0..7 -> 24..31.
    b       	testExpis0_2

whileExpNE0_2:
    add     	r2, r2, #8
    adds    	fpExp, fpExp, fpExp     // (LSL)
    bcc     	testExpis0_2
            
	add			r1, r7, r2
    vldr     	d1, [r1]
    vmul.f64    d0, d0, d1
            
testExpis0_2:
    cmp     fpExp, #0
    bne     whileExpNE0_2


// If the mantissa was negative, negate the result down here.

doMantissaSign:
    cmp     	sign, #false            //R3
    beq     	convSuccess
    
    vneg.f64    d0, d0

// Successful return here. Note: does not restore R0
// on successful conversion.
                    
convSuccess:
	add		r0, r0, #0	// clc = no error
	vpop	{d1}
	pop		{r1-r9, pc}
            
// Error returns down here. Returns error code in R1:

convError:  
	mov     r1, #-2
    b       ErrorExit

voor:       
	mov     r1, #-1 //Value out of range
    b       ErrorExit

illChar:    
	mov     r1, #-3

// Note: on error, this code restores X7.

ErrorExit:
	cmp		r1, #-3		// Sets the carry flag for error
	add		sp, sp, #4	// Don't restore R1
	vpop	{d1}
	pop		{r2-r9, pc}

    endp    bufToR64



			
