// bufToU32.S
//
// Unsigned decimal string to 32-bit integer 
// conversion.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2
	
	
////////////////////////////////////////////////////
//
// bufToI32
//
// Decimal string to 32-bit integer conversion.
// This code allows underscores to appear within a
// string (but underscores cannot be the first
// character of a string). This code also
// allows an underscore at the end of the string;
// if you consider this to be an error, modify
// the code at label "delimiter" to test for an
// underscore in the previous character position.
//
// On entry:
//
//	R0-		Pointer to buffer holding string.
//
// On exit-
//
//	R0-		Points at the first character beyond
//			the converted string, if conversion
//			was successful.
//
//	R1-		Contains the converted value if the
//			conversion was successful.
//
//	Carry-	Clear if conversion was successful.
//			Set if unsuccessful. If unsuccessful
//			conversion, then R0 will point at
//			the character the conversion failed
//			on and R1 will contain one of the
//			following error codes:
//
//			-1: First (non-WS) character was not
//				a valid decimal digit.
//
//			-2: Overflow (value did not fit into
//				32 bits).
//
//			-3: Last character was not a valid
//				delimiter character.
//
//	This function will skip any leading whitespace
// (spaces or tabs) appearing at the beginning
// of the string.
 			
	proc	bufToI32, thumb, public
	
	// Skip any leading whitespace characters
	// in the string:
	
	sub		r0, r0, #1		// Will adjust, below
skipWSI:
	ldrb	r1, [r0, #1]!
	
	cmp		r1, #' '
	beq		skipWSI
	cmp		r1, #tab
	beq		skipWSI
	cmp		r1, #','
	beq		skipWSI
	cmp		r1, #':'
	beq		skipWSI
	cmp		r1, #';'
	beq		skipWSI
	
	// See if the first character is a '-':
	
	cmp		r1, #'-'
	bne		bufToU32 	// No leading '-', use bufToU32.
	add		r0, r0, #1	// Skip '-'
	
	// If it was a negative number, call bufToU32
	// and then negate the result:
	
	push	{r2, lr}	// Push R2 to keep stk aligned
	bl		bufToU32
	bcs		iError
	negs	r1, r1
	bvs		iOverflow
	adds	r0, r0, #0	// Clear carry if no overflow
	pop		{r2, pc}
	
iOverflow:
	mov		r1, #-2
	cmp		r1, #-2		// Set the carry flag
iError:
	pop		{r2, pc}
	endp	bufToI32
		
	
	
////////////////////////////////////////////////////
//
// bufToU32
//
// Decimal string to 32-bit integer conversion.
// This code allows underscores to appear within a
// string (but underscores cannot be the first
// character of a string). This code also
// allows an underscore at the end of the string;
// if you consider this to be an error, modify
// the code at label "delimiter" to test for an
// underscore in the previous character position.
//
// On entry:
//
//	R0-		Pointer to buffer holding string.
//
// On exit-
//
//	R0-		Points at the first character beyond
//			the converted string, if conversion
//			was successful.
//
//	R1-		Contains the converted value if the
//			conversion was successful.
//
//	Carry-	Clear if conversion was successful.
//			Set if unsuccessful. If unsuccessful
//			conversion, then R0 will point at
//			the character the conversion failed
//			on and R1 will contain one of the
//			following error codes:
//
//			-1: First (non-WS) character was not
//				a valid decimal digit.
//
//			-2: Overflow (value did not fit into
//				32 bits).
//
//			-3: Last character was not a valid
//				delimiter character.
//
//	This function will skip any leading whitespace
// (spaces or tabs) appearing at the beginning
// of the string.

	proc	bufToU32, thumb, public
	push	{r2-r4, lr}
	
	mov		r1, #0			// Initialize result
	mov		r3, #10			// For multiply by 10
	
	// Skip any leading whitespace and delimiter
	// characters from the string:
	
	sub		r0, r0, #1		// Will adjust, below
skipWS:
	ldrb	r2, [r0, #1]!
	
	cmp		r2, #' '
	beq		skipWS
	cmp		r2, #tab
	beq		skipWS
	cmp		r2, #','
	beq		skipWS
	cmp		r2, #':'
	beq		skipWS
	cmp		r2, #';'
	beq		skipWS
	
	// Verify that the first character is a valid
	// decimal digit:
	
	cmp		r2, #'0'
	blo		badFirst
	cmp		r2, #'9'
	bls		decDigit
	
	// If the first digit is bad, return -1:
	
badFirst:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-1
	cmp		r1, r1				// Sets carry flag
	pop		{r2-r4, pc} 
		
	
	// If an overflow occurs, bail here:
	
overflow:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-2
	cmp		r1, r1				// Sets carry flag
	pop		{r2-r4, pc}

	
	// Transfer here if the character is
	// 0 to 9.
	
decDigit:
	and		r2, r2, #0xf		// Convert to integer
	umull	r1, r4, r1, r3		// R1*10
	tst		r4, r4				// Check for overflow
	bne		overflow
	add		r1, r1, r2			// Merge into result

skipChar:
	ldrb	r2, [r0, #1]!
	cmp		r2, #'0'
	blo		notDigit
	cmp		r2, #'9'
	bls		decDigit
	cmp		r2, #'_'
	beq		skipChar
	
notDigit:
	cmp		r2, #0				// Check for zero
	beq		delimiter
	cmp		r2, #' '
	beq		delimiter
	cmp		r2, #tab
	beq		delimiter
	cmp		r2, #','
	beq		delimiter
	cmp		r2, #':'
	beq		delimiter
	cmp		r2, #';'
	beq		delimiter
	cmp		r2, #0xa
	beq		delimiter
	cmp		r2, #0xd
	bne		illegal

	// Transfer here when encountering a legitimate
	// delimiter character (valid end of hex string).
	
delimiter:
	sub		r0, r0, #1			// Point at delimiter char
	adds	r0, r0, #0			// Clears carry flag
	pop		{r2-r4, pc}

	// If the first non-decimal-digit character
	// is bad, return -3:
	
illegal:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-3
	cmp		r1, r1				// Sets carry flag
	pop		{r2-r4, pc}

   	endp	bufToU32



			
