// bufToU64.S
//
// Unsigned decimal string to 64-bit integer 
// conversion.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2
	
	
////////////////////////////////////////////////////
//
// bufToI64
//
// Decimal string to 64-bit integer conversion.
// This code allows underscores to appear within a
// string (but underscores cannot be the first
// character of a string). This code also
// allows an underscore at the end of the string;
// if you consider this to be an error, modify
// the code at label "delimiter" to test for an
// underscore in the previous character position.
//
// On entry:
//
//	R0-		Pointer to buffer holding string.
//
// On exit-
//
//	R0-		Points at the first character beyond
//			the converted string, if conversion
//			was successful.
//
//	R2:R1-	Contains the converted value if the
//			conversion was successful.
//
//	Carry-	Clear if conversion was successful.
//			Set if unsuccessful. If unsuccessful
//			conversion, then R0 will point at
//			the character the conversion failed
//			on and R1 will contain one of the
//			following error codes:
//
//			-1: First (non-WS) character was not
//				a valid decimal digit.
//
//			-2: Overflow (value did not fit into
//				32 bits).
//
//			-3: Last character was not a valid
//				delimiter character.
//
//	This function will skip any leading whitespace
// (spaces or tabs) appearing at the beginning
// of the string.
 			
	proc	bufToI64, thumb, public
	
	// Skip any leading whitespace characters
	// in the string:
	
	sub		r0, r0, #1		// Will adjust, below
skipWSI:
	ldrb	r1, [r0, #1]!
	
	cmp		r1, #' '
	beq		skipWSI
	cmp		r1, #tab
	beq		skipWSI
	cmp		r1, #','
	beq		skipWSI
	cmp		r1, #':'
	beq		skipWSI
	cmp		r1, #';'
	beq		skipWSI
	
	// See if the first character is a '-':
	
	cmp		r1, #'-'
	bne		bufToU32 	// No leading '-', use bufToU32.
	add		r0, r0, #1	// Skip '-'
	
	// If it was a negative number, call bufToU32
	// and then negate the result:
	
	push	{r3, lr}	// Push R3 to keep stk aligned
	bl		bufToU64
	bcs		iError
	subs	r1, r7, r1	// Negate R2:r1
	sbcs	r2, r7, r2
	bvs		iOverflow
	adds	r0, r0, #0	// Clear carry if no overflow
	pop		{r3, pc}
	
iOverflow:
	mov		r1, #-2
	cmp		r1, #-2		// Set the carry flag
iError:
	pop		{r3, pc}
	endp	bufToI64
		
	
	
////////////////////////////////////////////////////
//
// bufToU64
//
// Decimal string to 64-bit integer conversion.
// This code allows underscores to appear within a
// string (but underscores cannot be the first
// character of a string). This code also
// allows an underscore at the end of the string;
// if you consider this to be an error, modify
// the code at label "delimiter" to test for an
// underscore in the previous character position.
//
// On entry:
//
//	R0-		Pointer to buffer holding string.
//
// On exit-
//
//	R0-		Points at the first character beyond
//			the converted string, if conversion
//			was successful.
//
//	R2:R1-	Contains the converted value if the
//			conversion was successful.
//
//	Carry-	Clear if conversion was successful.
//			Set if unsuccessful. If unsuccessful
//			conversion, then R0 will point at
//			the character the conversion failed
//			on and R1 will contain one of the
//			following error codes:
//
//			-1: First (non-WS) character was not
//				a valid decimal digit.
//
//			-2: Overflow (value did not fit into
//				32 bits).
//
//			-3: Last character was not a valid
//				delimiter character.
//
//	This function will skip any leading whitespace
// (spaces or tabs) appearing at the beginning
// of the string.

	proc	bufToU64, thumb, public
	push	{r3-r7, lr}
	
	mov		r1, #0			// Initialize result
	mov		r2, #0
	mov		r7, #0			// For reverse subtraction
	
	// Skip any leading whitespace and delimiter
	// characters from the string:
	
	sub		r0, r0, #1		// Will adjust, below
skipWS:
	ldrb	r3, [r0, #1]!
	
	cmp		r3, #' '
	beq		skipWS
	cmp		r3, #tab
	beq		skipWS
	cmp		r3, #','
	beq		skipWS
	cmp		r3, #':'
	beq		skipWS
	cmp		r3, #';'
	beq		skipWS
	
	// Verify that the first character is a valid
	// decimal digit:
	
	cmp		r3, #'0'
	blo		badFirst
	cmp		r3, #'9'
	bls		decDigit
	
	// If the first digit is bad, return -1:
	
badFirst:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-1
	cmp		r1, r1				// Sets carry flag
	pop		{r3-r7, pc} 
		
	
	// If an overflow occurs, bail here:
	
overflow:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-2
	cmp		r1, r1				// Sets carry flag
	pop		{r3-r7, pc}

	
	// Transfer here if the character is
	// 0 to 9.
	
decDigit:
	and		r3, r3, #0xf		// Convert to integer
	
	// Multiply R2:R1 by 10 and add in character:
	
	adds	r1, r1, r1			// R2:R1 * 2
	adcs	r2, r2, r2
	bcs		overflow	
	
	mov		r5, r1				// Save R2:R1 * 2
	mov		r6, r2

	adds	r1, r1, r1			// R2:R1 * 4
	adcs	r2, r2, r2
	bcs		overflow
		
	adds	r1, r1, r1			// R2:R1 * 8
	adcs	r2, r2, r2
	bcs		overflow	
		
	adds	r1, r1, r5			// R2:R1 * 10
	adcs	r2, r2, r6
	bcs		overflow	

	adds	r1, r1, r3			// Merge into result
	adcs	r2, r2, #0
	bcs		overflow

skipChar:
	ldrb	r3, [r0, #1]!
	cmp		r3, #'0'
	blo		notDigit
	cmp		r3, #'9'
	bls		decDigit
	cmp		r3, #'_'
	beq		skipChar
	
notDigit:
	cmp		r3, #0				// Check for zero
	beq		delimiter
	cmp		r3, #' '
	beq		delimiter
	cmp		r3, #tab
	beq		delimiter
	cmp		r3, #','
	beq		delimiter
	cmp		r3, #':'
	beq		delimiter
	cmp		r3, #';'
	beq		delimiter
	cmp		r3, #0xa
	beq		delimiter
	cmp		r3, #0xd
	bne		illegal

	// Transfer here when encountering a legitimate
	// delimiter character (valid end of hex string).
	
delimiter:
	sub		r0, r0, #1			// Point at delimiter char
	adds	r0, r0, #0			// Clears carry flag
	pop		{r3-r7, pc}

	// If the first non-decimal-digit character
	// is bad, return -3:
	
illegal:
	sub		r0, r0, #1			// Point R0 at bad char
	mov		r1, #-3
	cmp		r1, r1				// Sets carry flag
	pop		{r3-r7, pc}

   	endp	bufToU64



			
