// u32FmtStr.S
//
// u32FmtStr converts an unsigned 32-bit
// integer in R1 to a string of decimal
// digits, storing the string in the
// location pointed at by R0.
// This formatting version also outputs
// the string using a minimum field width
// and using a supplied fill character
// (if the number of digits is less than
// the field width).


	#include	"ao32aaLibPrivate.inc"

	.text
	.align  2
    
////////////////////////////////////////////////
//
//  u32FmtStr
//
//	Converts an integer to string of decimal
// digits using the minimum number of output
// positions. Zero-terminates the resulting
// string.
//
// Inputs:
//	R0: pointer to buffer (must have at least
//		11 bytes available in buffer).
//	R1: number to convert

	// Utility macro (just to save typing):
	
    .macro  divmod dest, src
    udiv    \dest, \src, r2
    mov     r4, \dest
    mls     r4, r4, r2, \src
    orr     r4, r4, #'0'
    strb    r4, [r0], #-1
    cmp     \dest, #0
    beq     uDone
    .endm
    
	
	// Special version if u32ToStr
	// that doesn't compute the
	// number of digits; instead,
	// this value is passed in R4
	// and R1 already points at the
	// last print position of the
	// output string.
	//
	// Note that the caller (u32FtmStr)
	// is responsible for setting up
	// the fill chars (if required).
	//
	// Also, this code does not preserve
	// registers, it is the caller's
	// responsibility to do that.
		
    proc    u32ToStrNoDigits, thumb, public
	
	// Special case for 0:
	
    cmp     r1, #0
    beq     u32ToStrNRSC    
	
	// Point R0 at the end of the buffer:
	
    add     r0, r0, r4

	// Emit the zero-terminating
	// byte:
	
    mov     r4, #0
    strb    r4, [r0], #-1    

	// R2 contains the constant used
	// by division and mod calculations:
	
    mov     r2, #10
    
	// Convert each digit, from LO to digit 9:
	
    divmod  r3, r1
    divmod  r1, r3
    divmod  r3, r1
    divmod  r1, r3
    divmod  r3, r1
    divmod  r1, r3        
    divmod  r3, r1        
    divmod  r1, r3
    divmod  r3, r1
    
    // Handle 10th digit specially
    
    udiv    r1, r3, r2
    mls     r4, r1, r2, r3
    orr     r4, r4, #'0'
    strb    r4, [r0], #-1

uDone:
    bx		lr 
    
    
    // Special case for 0:
    
u32ToStrNRSC:
    mov     r1, #'0'
    strb    r1, [r0]
    mov     r1, #0
    strb    r1, [r0, #1]        
    bx      lr
    endp    u32ToStrNoDigits



//////////////////////////////////////
//
// u32FtmStr
//
//	Converts a 32-bit unsigned integer
// to a string of decimal digits, with
// a minimum field width and a fill
// character (if the output string of
// digits will be less than the minimum
// field width).
//
// Inputs:
//	R0: Pointer to buffer to receive
//		the string (must be minimum
//		width + 1 bytes long, or 11,
//		whichever is greater).
//	R1:	Value to convert.
//	R2: Minimum field width
//  R3: Fill character (byte)

		proc	u32FmtStr, thumb, public
		push	{r0-r4, lr}
		
		// Compute the actual number of
		// digits in the number (u32Digits
		// returns result in R4):
		
		bl		u32Digits
		cmp		r4, r2
		bhi		noFillChars
		
// If the field width is greater than the number
// of digits in the converted value, then this
// code has to emit fill characters to make up
// the difference.
//
// In theory, this code could be sped up by
// writing four bytes at a time as long
// as there are still four fill characters
// to output. However, most widths are
// probably 12 characters or less and
// the overhead to output a word at a time
// probably would remove any benefit of
// doing this.

fillLp:
		strb	r3, [r0], #1
		subs	r2, r2, #1
		cmp		r2, r4
		bhi		fillLp
		
// At this point, R1 points at the first
// character position in the string.
// Call u32ToStrNoDigits to do the conversion.

noFillChars:
		bl		u32ToStrNoDigits
		
		pop		{r0-r4, pc}
		endp	u32FmtStr
		