// u64Digits.S
//
// u64Digits counts the number of decimal digits 
// required when converting a 64-bit unsigned
// integer to a string of decimal digits.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align  2

// u64Digits
//
// Input:
//  R2:R1-  Unsigned integer to process.
//
// Output:
//  R5: digit count
//
// For values less than 0x1_0000_0000 (that is,
// 32-bit values), this procedure calls u32Digits
// to compute the number of output positions. For
// values greater than this, this code uses a
// binary search over the following values:
//
// Digs 		  Decimal value		 Hexadecimal value
// --- --------------------------  ---------------------
// 11:             10,000,000,000 (0x0000_0002_540b_e400)
// 12:            100,000,000,000 (0x0000_0017_4876_E800)
// 13:          1,000,000,000,000 (0x0000_00E8_D4A5_1000)
// 14:         10,000,000,000,000 (0x0000_0918_4E72_A000)
// 15:        100,000,000,000,000 (0x0000_5AF3_107A_4000)
//
// 16:      1,000,000,000,000,000 (0x0003_8D7E_A4C6_8000)
// 17:     10,000,000,000,000,000 (0x0023_86F2_6FC1_0000)
// 18:    100,000,000,000,000,000 (0x0163_4578_5D8A_0000)
// 19:  1,000,000,000,000,000,000 (0x0DE0_B6B3_A764_0000)
// 20: 10,000,000,000,000,000,000 (0x8AC7_2304_89E8_0000)

    proc   u64Digits, thumb, public
    cmp     r2, #0
    bne     gt32Bits
    
    // If R1 is zero, transfer to u32Digits to process
    // the LO word:
    
	push	{r4, lr}
    bl       u32Digits
	mov		r5, r4
    pop		{r4, pc}

    // If R1 is non-zero, this is a bonafide 64-bit
    // value (well, at least greater than 32 bits).
    
gt32Bits:

    // Start by seeing if this number has 16 or more
    // digits:
    
    ldr     r5, =0x00038D7E // 1,000,000,000,000,000
    cmp     r2, r5
    bhi     try18
    blo     try13
    ldr     r5, =0xA4C68000 // LO word
    cmp     r1, r5
    bhs     try18
    
    // If less than 16 digits, see if it
    // has 13 or digits:
    
try13:
    ldr     r5, =0x000000E8 // 1,000,000,000,000
    cmp     r2, r5
    bhi     try15
    blo     try12
    ldr     r5, =0xD4A51000
    cmp     r1, r5
    bhs     try15
    
    // If fewer than 13 digits, see if it has 
	// 11 or 12 digits
    
try12:

    ldr     r5, =0x00000017 // 100,000,000,000
    cmp     r2, r5
    bhi     is12
	blo		is11
    ldr     r5, =0x4876E800
	cmp		r1, r5
    bhs     is12
	
	// Must be 11 digits.
	
is11:
	mov		r5, #11
	bx		lr
	
	// It's 12 digits:
	
is12:
	mov		r5, #12
	bx		lr
	
	// Convenient place to put is15:
	
is15:
	mov		r5, #15
	bx		lr
	
	// If 13 or more digits, see if it's
	// 13, 14, or 15
	
try15:
	ldr		r5, =0x00005AF3	// 100,000,000,000,000
	cmp		r2, r5
	bhi		is15
	blo		try14
	ldr		r5, =0x107A4000
	cmp		r1, r5
	bhs		is15
	
	// Must be 13 or 14 digits here:
	
try14:
	ldr		r5, =0x00000918 // 10,000,000,000,000
	cmp		r2, r5
	bhi		is14
	blo		is13
	ldr		r5, =0x4E72A000
	cmp		r1, r5
	bhs		is14
	
	// Must be 13 digits, here:
	
is13:
	mov		r5, #13
	bx		lr
	
	// Is 14 digits here:
	
is14:
	mov		r5, #14
	bx		lr
	

	// Handle 16-20 digits down here:

try18:
	ldr		r5, =0x01634578	// 100,000,000,000,000,000
	cmp		r2, r5
	bhi		try19
	blo		try17
	ldr		r5, =0x5D8A0000
	cmp		r1, r5
	bhs		try19
	
	// Handle 16 or 17 digits here
	
try17:
	ldr		r5, =0x002386F2 // 10,000,000,000,000,000
	cmp		r2, r5
	bhi		is17
	blo		is16
	ldr		r5, =0x6FC10000 
	cmp		r1, r5
	bhs		is17
	
	// Must be 16 digits at this point
	
is16:
	mov		r5, #16
	bx		lr
	
	// Is 17 digits at this point:
	
is17:
	mov		r5, #17
	bx		lr
	
	// Handle 18-20 digits here:
	
try19:
	ldr		r5, =0x0DE0B6B3 // 1,000,000,000,000,000,000
	cmp		r2, r5
	bhi		try20
	blo		is18
	ldr		r5, =0xA7640000   
	cmp		r1, r5
	bhs		try20
	
	// Must be 18 digits at this point:
	
is18:
	mov		r5, #18
	bx		lr
	
	// Handle 19-20 digits here.
	
try20:
	ldr		r5, =0x8AC72304 // 10,000,000,000,000,000,000
	cmp		r2, r5
	bhi		is20
	blo		is19
	ldr		r5, =0x89E80000
	cmp		r1, r5
	bhs		is20
	
	// Must be 19 digits here:
	
is19:
	mov		r5, #19
	bx		lr
	
	// Is 20 digits here.
	
is20:
	mov		r5, #20
	bx		lr
    endp    u64Digits  
