// u64ToBuf.S
//
// u64ToBuf converts an unsigned 32-bit
// integer in R2:R1 to a string of decimal
// digits.

	#include	"ao32aaLibPrivate.inc"

	.text				 
	.align  2

////////////////////////////////////////////////
//
//  u64ToBuf
//
//	Converts an integer to string of decimal
// digits using the minimum number of output
// positions.
//
// Inputs:
//	R0-		Pointer to the buffer.
//	R2:R1-	Number to convert.
//
// Output:
//	R0: points at the first byte after
//		the string of converted digits.
//
// Does not preserve any registers! Caller is
// resposible for saving R0-R5, if necessary.
//
// Note that this procedure does not zero-terminate
// the string.

    proc    u64ToBuf, thumb, public
	push	{lr}
	
	// Compute the number of output positions this
	// number requires, and point R0 at the end of
	// the buffer where the least significant digit
	// will go:
		
	bl		u64Digits
	add		r0, r0, r5	// Point at end of buffer+1
	push	{r0}		// Preserve r0
	sub		r0, r0, #1	// Point at last char posn
	
	// Convert the string to digits:
	
	bl		u64ToBufRev
	
	// Because R0 was pushed last, it is at the
	// lowest address on the stack and the following
	// pop properly puts TOS into R0:
	
	pop		{r0, pc}
	endp	u64ToBuf
	
////////////////////////////////////////////////////////////
//
// divMod64by10
//
//	Divides value passed in R2:R1 by 10.
//	Returns quotient in R2:R1, remainder in R3.
//  Does this by multipling by 0.1 so no division is involved.
//
// see http://homepage.cs.uiowa.edu/~jones/bcd/divide.html
//
//	
// 64-bit multiply:
//
// 		  	  HO     LO
//  		  AAA : BBB (r2:r1)
//  		x CCC : DDD (0x00c...ccd)
//       -------------
//             | D*B |
//          | D*A |
//          | C*B |
//     + | C*A |
// -------------------
// | AAABBB * CCCDDD |
//
// Note: CCC:DDD =
//  0xcccccccc:0xcccccccd
// which is 0.1 in fixed-point
// notation.
//
// Note: this function overwrites the
// values in R0 to R11 (except R0). 
// It is the caller's responsibility 
// to preserve these registers, if
// necessary.
	
	proc	divMod64by10, thumb, public

	// R4:R5 = D*B
	
	ldr		r3, =0xcccccccd	// D
	umull	r4, r5, r1, r3
	
	// R6:R7 = D*A
	
	umull	r6, r7, r2, r3
	
	// R8:R9 = C*B
	
	ldr		r3, =0xcccccccc	// C
	umull	r8, r9, r1, r3	
	
	// R10:R11 = C*A
	
	umull	r10, r11, r2, r3
		
	// Now add together the partial
	// products. Note that the
	// LO word of the partial products
	// does not affect the result,
	// so this code doesn't add it in.
	
	adds	r5, r5, r6
	adcs	r7, r7, r9
	adc		r11, r11, #0
	
	adds	r5, r5, r8
	adcs	r7, r7, r10
	adcs	r11, r11, #0
		
	// r2:r1 * 10 * 0x1000_0000 * 8 is sitting in
	// R11:R7. Therefore, R10 holds the quotient * 8.
	
	lsr		r7, r7, #3
	orr		r7, r7, r11, lsl #29
	lsr		r11, r11, #3
	
	// Now, multiply the quotient by 10 and
	// subtract it from the original value (r2:r1)
	// to get the remainder:
	
	mov		r3, #10
	mls		r3, r7, r3, r1
	
	mov		r1, r7
	mov		r2, r11
	
	bx		lr
	endp	divMod64by10
	

////////////////////////////////////////////////
//
//  u64ToBufRev
//
//	Converts a 64-bit unsigned integer to string
// of decimal digits using the minimum number of
// output positions.
//
// Inputs:
//	r2:r1-	Number to convert
//	R0-		Pointer to the end of the buffer
//      	where the least significant digit
//      	will go.
//
// Output:
//	R0-		Points at the first byte before
//			the string of converted digits.
//
// Does not preserve any registers! Caller
// is responsible for preserving R0-R11, as
// needed.
//
// Note that this procedure does not zero-terminate
// the string.

	// Utility macro (just to save typing):
	
    .macro  divmod
	bl		divMod64by10
	orr		r3, r3, #'0'
	strb	r3, [r0], #-1
	orrs	r3, r1, r2	// See if quotient is 0
    beq     uDone
    .endm
	    
	
	// Conversion function:
	
    proc    u64ToBufRev, thumb, public
	push	{lr}
    
	// Convert each digit, from LO to digit 9:
	
    divmod		// LO digit
    divmod		// Digit 1
    divmod		// Digit 2
    divmod		// Digit 3
    divmod		// Digit 4
    divmod		// Digit 5
    divmod		// Digit 6
    divmod		// Digit 7
    divmod		// Digit 8
    divmod		// Digit 9
    divmod		// Digit 10
    divmod		// Digit 11
    divmod		// Digit 12
    divmod		// Digit 13
    divmod		// Digit 14
    divmod		// Digit 15
    divmod		// Digit 16
    divmod		// Digit 17
    divmod		// Digit 18
    
    // Handle HO digit (19) specially:
    
	bl		divMod64by10
	orr		r3, r3, #'0'
	strb	r3, [r0], #-1

uDone:
    pop		{pc} 
    endp    u64ToBufRev



